@@ -1044,7 +1044,7 @@ int CRealControlSocket::ContinueConnect()
 
 	const int proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
 	if (proxy_type > CProxySocket::unknown && proxy_type < CProxySocket::proxytype_count && !m_pCurrentServer->GetBypassProxy()) {
-		LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), m_pCurrentServer->FormatHost(), CProxySocket::Name(static_cast<CProxySocket::ProxyType>(proxy_type)));
+		LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), m_pCurrentServer->Format(ServerFormat::with_optional_port), CProxySocket::Name(static_cast<CProxySocket::ProxyType>(proxy_type)));
 
 		host = engine_.GetOptions().GetOption(OPTION_PROXY_HOST);
 		port = engine_.GetOptions().GetOptionVal(OPTION_PROXY_PORT);
@@ -1285,16 +1285,21 @@ void CControlSocket::CreateLocalDir(const wxString &local_file)
 	for (auto iter = segments.rbegin(); iter != segments.rend(); ++iter) {
 		local_path.AddSegment(*iter);
 
-#ifdef __WXMSW__
-		BOOL res = CreateDirectory(local_path.GetPath().wc_str(), 0);
-		if (!res && GetLastError() != ERROR_ALREADY_EXISTS)
+#ifdef FZ_WINDOWS
+		BOOL res = CreateDirectory(local_path.GetPath().c_str(), 0);
+		if (!res && GetLastError() != ERROR_ALREADY_EXISTS) {
 			break;
+		}
 #else
-		const wxCharBuffer s = local_path.GetPath().fn_str();
-
-		int res = mkdir(s, 0777);
-		if (res && errno != EEXIST)
+		fz::native_string s = fz::to_native(local_path.GetPath());
+		if (s.empty()) {
 			break;
+		}
+
+		int res = mkdir(s.c_str(), 0777);
+		if (res && errno != EEXIST) {
+			break;
+		}
 #endif
 		last_successful = local_path;
 	}
