@@ -26,7 +26,7 @@ CDirectoryCache::~CDirectoryCache()
 
 void CDirectoryCache::Store(const CDirectoryListing &listing, const CServer &server)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = CreateServerEntry(server);
 	wxASSERT(sit != m_serverList.end());
@@ -36,24 +36,25 @@ void CDirectoryCache::Store(const CDirectoryListing &listing, const CServer &ser
 	tCacheIter cit;
 	bool unused;
 	if (Lookup(cit, sit, listing.path, true, unused)) {
-		cit->modificationTime = CMonotonicTime::Now();
+		auto & entry = const_cast<CCacheEntry&>(*cit);
+		entry.modificationTime = fz::monotonic_clock::now();
 
 		m_totalFileCount -= cit->listing.GetCount();
-		cit->listing = listing;
+		entry.listing = listing;
 
 		return;
 	}
 
-	sit->cacheList.emplace_front(listing);
+	cit = sit->cacheList.emplace_hint(cit, listing);
 
-	UpdateLru(sit, sit->cacheList.begin());
+	UpdateLru(sit, cit);
 
 	Prune();
 }
 
 bool CDirectoryCache::Lookup(CDirectoryListing &listing, const CServer &server, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
 	if (sit == m_serverList.end())
@@ -70,20 +71,22 @@ bool CDirectoryCache::Lookup(CDirectoryListing &listing, const CServer &server,
 
 bool CDirectoryCache::Lookup(tCacheIter &cacheIter, tServerIter &sit, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated)
 {
-	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
-		const CCacheEntry &entry = *iter;
+	static CCacheEntry dummy;
+	dummy.listing.path = path;
+	cacheIter = sit->cacheList.lower_bound(dummy);
 
-		if (entry.listing.path != path)
-			continue;
+	if (cacheIter != sit->cacheList.end()) {
+		CCacheEntry const& entry = *cacheIter;
 
-		UpdateLru(sit, iter);
+		if (entry.listing.path == path) {
+			UpdateLru(sit, cacheIter);
 
-		if (!allowUnsureEntries && entry.listing.get_unsure_flags())
-			return false;
+			if (!allowUnsureEntries && entry.listing.get_unsure_flags())
+				return false;
 
-		cacheIter = iter;
-		is_outdated = (CDateTime::Now() - entry.listing.m_firstListTime.GetTime()).GetSeconds() > CACHE_TIMEOUT;
-		return true;
+			is_outdated = (fz::monotonic_clock::now() - entry.listing.m_firstListTime).get_seconds() > CACHE_TIMEOUT;
+			return true;
+		}
 	}
 
 	return false;
@@ -91,7 +94,7 @@ bool CDirectoryCache::Lookup(tCacheIter &cacheIter, tServerIter &sit, const CSer
 
 bool CDirectoryCache::DoesExist(const CServer &server, const CServerPath &path, int &hasUnsureEntries, bool &is_outdated)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
 	if (sit == m_serverList.end())
@@ -108,7 +111,7 @@ bool CDirectoryCache::DoesExist(const CServer &server, const CServerPath &path,
 
 bool CDirectoryCache::LookupFile(CDirentry &entry, const CServer &server, const CServerPath &path, const wxString& file, bool &dirDidExist, bool &matchedCase)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
 	if (sit == m_serverList.end()) {
@@ -143,32 +146,16 @@ bool CDirectoryCache::LookupFile(CDirentry &entry, const CServer &server, const
 	return false;
 }
 
-CDirectoryCache::CCacheEntry& CDirectoryCache::CCacheEntry::operator=(const CDirectoryCache::CCacheEntry &a)
-{
-	lruIt = a.lruIt;
-	listing = a.listing;
-	modificationTime = a.modificationTime;
-
-	return *this;
-}
-
-CDirectoryCache::CCacheEntry::CCacheEntry(const CDirectoryCache::CCacheEntry &entry)
-	: lruIt()
-{
-	listing = entry.listing;
-	modificationTime = entry.modificationTime;
-}
-
 bool CDirectoryCache::InvalidateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool *wasDir /*=false*/)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
 	if (sit == m_serverList.end())
 		return false;
 
 	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
-		CCacheEntry &entry = *iter;
+		auto & entry = const_cast<CCacheEntry&>(*iter);
 		if (path.CmpNoCase(entry.listing.path))
 			continue;
 
@@ -182,15 +169,15 @@ bool CDirectoryCache::InvalidateFile(const CServer &server, const CServerPath &p
 			}
 		}
 		entry.listing.m_flags |= CDirectoryListing::unsure_unknown;
-		entry.modificationTime = CMonotonicTime::Now();
+		entry.modificationTime = fz::monotonic_clock::now();
 	}
 
 	return true;
 }
 
-bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool mayCreate, enum Filetype type /*=file*/, wxLongLong size /*=-1*/)
+bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path, const wxString& filename, bool mayCreate, enum Filetype type /*=file*/, int64_t size /*=-1*/)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
 	if (sit == m_serverList.end())
@@ -198,10 +185,8 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 
 	bool updated = false;
 
-	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter)
-	{
-		CCacheEntry &entry = *iter;
-		const CCacheEntry &cEntry = *iter;
+	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
+		auto & entry = const_cast<CCacheEntry&>(*iter);
 		if (path.CmpNoCase(entry.listing.path))
 			continue;
 
@@ -209,21 +194,17 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 
 		bool matchCase = false;
 		unsigned int i;
-		for (i = 0; i < entry.listing.GetCount(); i++)
-		{
-			if (!filename.CmpNoCase(cEntry.listing[i].name))
-			{
+		for (i = 0; i < entry.listing.GetCount(); ++i) {
+			if (!filename.CmpNoCase(entry.listing[i].name)) {
 				entry.listing[i].flags |= CDirentry::flag_unsure;
-				if (cEntry.listing[i].name == filename)
-				{
+				if (entry.listing[i].name == filename) {
 					matchCase = true;
 					break;
 				}
 			}
 		}
 
-		if (matchCase)
-		{
+		if (matchCase) {
 			enum Filetype old_type = entry.listing[i].is_dir() ? dir : file;
 			if (type != old_type)
 				entry.listing.m_flags |= CDirectoryListing::unsure_invalid;
@@ -232,21 +213,18 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 			else
 				entry.listing.m_flags |= CDirectoryListing::unsure_file_changed;
 		}
-		else if (type != unknown && mayCreate)
-		{
+		else if (type != unknown && mayCreate) {
 			const unsigned int count = entry.listing.GetCount();
-			entry.listing.SetCount(count + 1);
-			CDirentry& direntry = entry.listing[count];
+			CDirentry direntry;
 			direntry.name = filename;
 			if (type == dir)
 				direntry.flags = CDirentry::flag_dir | CDirentry::flag_unsure;
 			else
 				direntry.flags = CDirentry::flag_unsure;
 			direntry.size = size;
-			switch (type)
-			{
+			switch (type) {
 			case dir:
-				entry.listing.m_flags |= CDirectoryListing::unsure_dir_added;
+				entry.listing.m_flags |= CDirectoryListing::unsure_dir_added | CDirectoryListing::listing_has_dirs;
 				break;
 			case file:
 				entry.listing.m_flags |= CDirectoryListing::unsure_file_added;
@@ -255,12 +233,13 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 				entry.listing.m_flags |= CDirectoryListing::unsure_invalid;
 				break;
 			}
+			entry.listing.Append(std::move(direntry));
 
 			++m_totalFileCount;
 		}
 		else
 			entry.listing.m_flags |= CDirectoryListing::unsure_unknown;
-		entry.modificationTime = CMonotonicTime::Now();
+		entry.modificationTime = fz::monotonic_clock::now();
 
 		updated = true;
 	}
@@ -270,49 +249,43 @@ bool CDirectoryCache::UpdateFile(const CServer &server, const CServerPath &path,
 
 bool CDirectoryCache::RemoveFile(const CServer &server, const CServerPath &path, const wxString& filename)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
 	if (sit == m_serverList.end())
 		return false;
 
-	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter)
-	{
-		const CCacheEntry &entry = *iter;
+	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ++iter) {
+		auto & entry = const_cast<CCacheEntry&>(*iter);
 		if (path.CmpNoCase(entry.listing.path))
 			continue;
 
 		UpdateLru(sit, iter);
 
 		bool matchCase = false;
-		for (unsigned int i = 0; i < entry.listing.GetCount(); i++)
-		{
+		for (unsigned int i = 0; i < entry.listing.GetCount(); ++i) {
 			if (entry.listing[i].name == filename)
 				matchCase = true;
 		}
 
-		if (matchCase)
-		{
+		if (matchCase) {
 			unsigned int i;
 			for (i = 0; i < entry.listing.GetCount(); i++)
 				if (entry.listing[i].name == filename)
 					break;
 			wxASSERT(i != entry.listing.GetCount());
 
-			CDirectoryListing& listing = iter->listing;
-			listing.RemoveEntry(i); // This does set m_hasUnsureEntries
+			entry.listing.RemoveEntry(i); // This does set m_hasUnsureEntries
 			--m_totalFileCount;
 		}
-		else
-		{
-			for (unsigned int i = 0; i < entry.listing.GetCount(); i++)
-			{
+		else {
+			for (unsigned int i = 0; i < entry.listing.GetCount(); ++i) {
 				if (!filename.CmpNoCase(entry.listing[i].name))
-					iter->listing[i].flags |= CDirentry::flag_unsure;
+					entry.listing[i].flags |= CDirentry::flag_unsure;
 			}
-			iter->listing.m_flags |= CDirectoryListing::unsure_invalid;
+			entry.listing.m_flags |= CDirectoryListing::unsure_invalid;
 		}
-		iter->modificationTime = CMonotonicTime::Now();
+		entry.modificationTime = fz::monotonic_clock::now();
 	}
 
 	return true;
@@ -320,7 +293,7 @@ bool CDirectoryCache::RemoveFile(const CServer &server, const CServerPath &path,
 
 void CDirectoryCache::InvalidateServer(const CServer& server)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
 	{
@@ -344,9 +317,9 @@ void CDirectoryCache::InvalidateServer(const CServer& server)
 	}
 }
 
-bool CDirectoryCache::GetChangeTime(CMonotonicTime& time, const CServer &server, const CServerPath &path)
+bool CDirectoryCache::GetChangeTime(fz::monotonic_clock& time, const CServer &server, const CServerPath &path)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
 	if (sit == m_serverList.end())
@@ -364,7 +337,7 @@ bool CDirectoryCache::GetChangeTime(CMonotonicTime& time, const CServer &server,
 
 void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path, const wxString& filename, const CServerPath&)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	// TODO: This is not 100% foolproof and may not work properly
 	// Perhaps just throw away the complete cache?
@@ -378,7 +351,7 @@ void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path,
 		absolutePath.clear();
 
 	for (tCacheIter iter = sit->cacheList.begin(); iter != sit->cacheList.end(); ) {
-		CCacheEntry &entry = *iter;
+		auto & entry = const_cast<CCacheEntry&>(*iter);
 		// Delete exact matches and subdirs
 		if (!absolutePath.empty() && (entry.listing.path == absolutePath || absolutePath.IsParentOf(entry.listing.path, true))) {
 			m_totalFileCount -= entry.listing.GetCount();
@@ -399,7 +372,7 @@ void CDirectoryCache::RemoveDir(const CServer& server, const CServerPath& path,
 
 void CDirectoryCache::Rename(const CServer& server, const CServerPath& pathFrom, const wxString& fileFrom, const CServerPath& pathTo, const wxString& fileTo)
 {
-	scoped_lock lock(mutex_);
+	fz::scoped_lock lock(mutex_);
 
 	tServerIter sit = GetServerEntry(server);
 	if (sit == m_serverList.end())
@@ -408,9 +381,8 @@ void CDirectoryCache::Rename(const CServer& server, const CServerPath& pathFrom,
 	tCacheIter iter;
 	bool is_outdated = false;
 	bool found = Lookup(iter, sit, pathFrom, true, is_outdated);
-	if (found)
-	{
-		CDirectoryListing& listing = iter->listing;
+	if (found) {
+		auto & listing = const_cast<CDirectoryListing&>(iter->listing);
 		if (pathFrom == pathTo)
 		{
 			RemoveFile(server, pathFrom, fileTo);
@@ -491,8 +463,10 @@ void CDirectoryCache::UpdateLru(tServerIter const& sit, tCacheIter const& cit)
 		m_leastRecentlyUsedList.splice(m_leastRecentlyUsedList.end(), m_leastRecentlyUsedList, *lruIt);
 		**lruIt = std::make_pair(sit, cit);
 	}
-	else
-		cit->lruIt = (void*)new tLruList::iterator(m_leastRecentlyUsedList.emplace(m_leastRecentlyUsedList.end(), sit, cit));
+	else {
+		auto & entry = const_cast<CCacheEntry&>(*cit);
+		entry.lruIt = (void*)new tLruList::iterator(m_leastRecentlyUsedList.emplace(m_leastRecentlyUsedList.end(), sit, cit));
+	}
 }
 
 void CDirectoryCache::Prune()
