@@ -59,11 +59,11 @@ public:
 	CDirectoryListing Parse(const CServerPath &path);
 
 	bool AddData(char *pData, int len);
-	bool AddLine(const wxChar* pLine);
+	bool AddLine(const wchar_t* pLine);
 
 	void Reset();
 
-	void SetTimezoneOffset(duration const& span) { m_timezoneOffset = span; }
+	void SetTimezoneOffset(fz::duration const& span) { m_timezoneOffset = span; }
 
 	void SetServer(const CServer& server) { m_server = server; };
 
@@ -72,7 +72,7 @@ protected:
 
 	bool ParseData(bool partial);
 
-	bool ParseLine(CLine &line, const enum ServerType serverType, bool concatenated);
+	bool ParseLine(CLine &line, ServerType const serverType, bool concatenated);
 
 	bool ParseAsUnix(CLine &line, CDirentry &entry, bool expect_date);
 	bool ParseAsDos(CLine &line, CDirentry &entry);
@@ -103,16 +103,16 @@ protected:
 	bool ParseTime(CToken &token, CDirentry &entry);
 
 	// Parse file sizes given like this: 123.4M
-	bool ParseComplexFileSize(CToken& token, wxLongLong& size, int blocksize = -1);
+	bool ParseComplexFileSize(CToken& token, int64_t& size, int blocksize = -1);
 
-	bool GetMonthFromName(const wxString& name, int &month);
+	bool GetMonthFromName(std::wstring const& name, int &month);
 
 	void DeduceEncoding();
 	void ConvertEncoding(char *pData, int len);
 
 	CControlSocket* m_pControlSocket;
 
-	static std::map<wxString, int> m_MonthNamesMap;
+	static std::map<std::wstring, int> m_MonthNamesMap;
 
 	struct t_list
 	{
@@ -128,8 +128,8 @@ protected:
 	int m_currentOffset;
 
 	std::deque<t_list> m_DataList;
-	std::deque<CRefcountObject<CDirentry>> m_entryList;
-	wxLongLong m_totalData;
+	std::deque<fz::shared_value<CDirentry>> m_entryList;
+	int64_t m_totalData;
 
 	CLine *m_prevLine;
 
@@ -140,7 +140,7 @@ protected:
 
 	bool m_maybeMultilineVms;
 
-	duration m_timezoneOffset;
+	fz::duration m_timezoneOffset;
 
 	listingEncoding::type m_listingEncoding;
 
