@@ -1,13 +1,13 @@
 #ifndef __FILEZILLAENGINEPRIVATE_H__
 #define __FILEZILLAENGINEPRIVATE_H__
 
-#include "timeex.h"
+#include <libfilezilla/event.hpp>
+#include <libfilezilla/event_handler.hpp>
+#include <libfilezilla/mutex.hpp>
+#include <libfilezilla/time.hpp>
 
 #include "engine_context.h"
-#include "event.h"
-#include "event_handler.h"
 #include "FileZillaEngine.h"
-#include "mutex.h"
 #include "option_change_event_handler.h"
 
 #include <atomic>
@@ -23,7 +23,7 @@ enum EngineNotificationType
 };
 
 struct filezilla_engine_event_type;
-typedef CEvent<filezilla_engine_event_type, EngineNotificationType> CFileZillaEngineEvent;
+typedef fz::simple_event<filezilla_engine_event_type, EngineNotificationType> CFileZillaEngineEvent;
 
 class CTransferStatusManager final
 {
@@ -44,7 +44,7 @@ public:
 	CTransferStatus Get(bool &changed);
 
 protected:
-	mutex mutex_;
+	fz::mutex mutex_;
 
 	CTransferStatus status_;
 	std::atomic<int64_t> currentOffset_{};
@@ -53,14 +53,12 @@ protected:
 	CFileZillaEnginePrivate& engine_;
 };
 
-class CFileZillaEnginePrivate final : public CEventHandler, COptionChangeEventHandler
+class CFileZillaEnginePrivate final : public fz::event_handler, COptionChangeEventHandler
 {
 public:
-	CFileZillaEnginePrivate(CFileZillaEngineContext& engine_context, CFileZillaEngine& parent);
+	CFileZillaEnginePrivate(CFileZillaEngineContext& engine_context, CFileZillaEngine& parent, EngineNotificationHandler& notificationHandler);
 	virtual ~CFileZillaEnginePrivate();
 
-	int Init(wxEvtHandler *pEventHandler);
-
 	int Execute(CCommand const& command);
 	int Cancel();
 	int ResetOperation(int nErrorCode);
@@ -135,19 +133,19 @@ protected:
 
 	int ContinueConnect();
 
-	void operator()(CEventBase const& ev);
+	void operator()(fz::event_base const& ev);
 	void OnEngineEvent(EngineNotificationType type);
 	void OnTimer(int timer_id);
 	void OnCommandEvent();
 
 	// General mutex for operations on the engine
 	// Todo: More fine-grained locking, a global mutex isn't nice
-	static mutex mutex_;
+	static fz::mutex mutex_;
 
 	// Used to synchronize access to the notification list
-	mutex notification_mutex_;
+	fz::mutex notification_mutex_;
 
-	wxEvtHandler *m_pEventHandler{};
+	EngineNotificationHandler& notification_handler_;
 
 	int m_engine_id;
 
@@ -158,7 +156,7 @@ protected:
 
 	// Remember last path used in a dirlisting.
 	CServerPath m_lastListDir;
-	CMonotonicTime m_lastListTime;
+	fz::monotonic_clock m_lastListTime;
 
 	std::unique_ptr<CControlSocket> m_pControlSocket;
 
@@ -187,12 +185,12 @@ protected:
 	struct t_failedLogins final
 	{
 		CServer server;
-		wxDateTime time;
+		fz::datetime time;
 		bool critical{};
 	};
 	static std::list<t_failedLogins> m_failedLogins;
 	int m_retryCount{};
-	timer_id m_retryTimer{};
+	fz::timer_id m_retryTimer{};
 
 	CRateLimiter& m_rateLimiter;
 	CDirectoryCache& directory_cache_;
@@ -205,9 +203,9 @@ protected:
 };
 
 struct command_event_type{};
-typedef CEvent<command_event_type> CCommandEvent;
+typedef fz::simple_event<command_event_type> CCommandEvent;
 
 struct async_request_reply_event_type{};
-typedef CEvent<async_request_reply_event_type, std::unique_ptr<CAsyncRequestNotification>> CAsyncRequestReplyEvent;
+typedef fz::simple_event<async_request_reply_event_type, std::unique_ptr<CAsyncRequestNotification>> CAsyncRequestReplyEvent;
 
 #endif //__FILEZILLAENGINEPRIVATE_H__
