@@ -1,17 +1,20 @@
 #include <filezilla.h>
 #include "externalipresolver.h"
-#include "wx/regex.h"
 #include "socket.h"
 #include "misc.h"
 
+#include <libfilezilla/iputils.hpp>
+
+#include <wx/regex.h>
+
 namespace {
-mutex s_sync;
+fz::mutex s_sync;
 wxString ip;
 bool checked = false;
 }
 
-CExternalIPResolver::CExternalIPResolver(CEventHandler & handler)
-	: CEventHandler(handler.event_loop_)
+CExternalIPResolver::CExternalIPResolver(event_handler & handler)
+	: event_handler(handler.event_loop_)
 	, m_handler(&handler)
 {
 	ResetHttpData(true);
@@ -19,6 +22,8 @@ CExternalIPResolver::CExternalIPResolver(CEventHandler & handler)
 
 CExternalIPResolver::~CExternalIPResolver()
 {
+	remove_handler();
+
 	delete [] m_pSendBuffer;
 	m_pSendBuffer = 0;
 	delete [] m_pRecvBuffer;
@@ -31,7 +36,7 @@ CExternalIPResolver::~CExternalIPResolver()
 void CExternalIPResolver::GetExternalIP(const wxString& address, CSocket::address_family protocol, bool force /*=false*/)
 {
 	{
-		scoped_lock l(s_sync);
+		fz::scoped_lock l(s_sync);
 		if (checked) {
 			if (force)
 				checked = false;
@@ -73,7 +78,7 @@ void CExternalIPResolver::GetExternalIP(const wxString& address, CSocket::addres
 
 	m_pSocket = new CSocket(this);
 
-	int res = m_pSocket->Connect(host, m_port, protocol);
+	int res = m_pSocket->Connect(fz::to_native(host), m_port, protocol);
 	if (res && res != EINPROGRESS) {
 		Close(false);
 		return;
@@ -84,9 +89,9 @@ void CExternalIPResolver::GetExternalIP(const wxString& address, CSocket::addres
 	strcpy(m_pSendBuffer, buffer.mb_str());
 }
 
-void CExternalIPResolver::operator()(CEventBase const& ev)
+void CExternalIPResolver::operator()(fz::event_base const& ev)
 {
-	Dispatch<CSocketEvent>(ev, this, &CExternalIPResolver::OnSocketEvent);
+	fz::dispatch<CSocketEvent>(ev, this, &CExternalIPResolver::OnSocketEvent);
 }
 
 void CExternalIPResolver::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
@@ -217,7 +222,7 @@ void CExternalIPResolver::Close(bool successful)
 	m_done = true;
 
 	{
-		scoped_lock l(s_sync);
+		fz::scoped_lock l(s_sync);
 		if (!successful) {
 			ip.clear();
 		}
@@ -225,7 +230,7 @@ void CExternalIPResolver::Close(bool successful)
 	}
 
 	if (m_handler) {
-		m_handler->SendEvent<CExternalIPResolveEvent>();
+		m_handler->send_event<CExternalIPResolveEvent>();
 		m_handler = 0;
 	}
 }
@@ -373,12 +378,12 @@ void CExternalIPResolver::OnData(char* buffer, unsigned int len)
 			m_data = m_data.Mid(1);
 		}
 
-		if (GetIPV6LongForm(m_data).empty()) {
+		if (fz::get_ipv6_long_form(m_data.ToStdWstring()).empty()) {
 			Close(false);
 			return;
 		}
 
-		scoped_lock l(s_sync);
+		fz::scoped_lock l(s_sync);
 		ip = m_data;
 	}
 	else {
@@ -395,7 +400,7 @@ void CExternalIPResolver::OnData(char* buffer, unsigned int len)
 			return;
 		}
 
-		scoped_lock l(s_sync);
+		fz::scoped_lock l(s_sync);
 		ip = regex.GetMatch(m_data, 2);
 	}
 
@@ -413,9 +418,7 @@ void CExternalIPResolver::ResetHttpData(bool resetRedirectCount)
 
 	m_transferEncoding = unknown;
 
-	m_chunkData.getTrailer = false;
-	m_chunkData.size = 0;
-	m_chunkData.terminateChunk = false;
+	m_chunkData = t_chunkData();
 
 	m_finished = false;
 }
@@ -429,7 +432,7 @@ void CExternalIPResolver::OnChunkedData()
 		if (m_chunkData.size != 0) {
 			unsigned int dataLen = len;
 			if (m_chunkData.size < len)
-				dataLen = m_chunkData.size.GetLo();
+				dataLen = static_cast<unsigned int>(m_chunkData.size);
 			OnData(p, dataLen);
 			if (!m_pRecvBuffer)
 				return;
@@ -528,12 +531,12 @@ void CExternalIPResolver::OnChunkedData()
 
 bool CExternalIPResolver::Successful() const
 {
-	scoped_lock l(s_sync);
+	fz::scoped_lock l(s_sync);
 	return !ip.empty();
 }
 
 wxString CExternalIPResolver::GetIP() const
 {
-	scoped_lock l(s_sync);
+	fz::scoped_lock l(s_sync);
 	return ip;
 }
