@@ -94,7 +94,7 @@ protected:
 	// It's the last line in a multi-line response.
 	void ParseResponse();
 
-	void ParseFeat(wxString const& line);
+	void ParseFeat(wxString line);
 
 	virtual int SendNextCommand();
 	virtual int ParseSubcommandResult(int prevResult);
@@ -114,7 +114,7 @@ protected:
 	// Other servers return "550 No files found."
 	bool IsMisleadingListResponse() const;
 
-	int GetExternalIPAddress(wxString& address);
+	int GetExternalIPAddress(std::string& address);
 
 	// Checks if listing2 is a subset of listing1. Compares only filenames.
 	bool CheckInclusion(const CDirectoryListing& listing1, const CDirectoryListing& listing2);
@@ -148,16 +148,16 @@ protected:
 
 	// Used by keepalive code so that we're not using keep alive
 	// till the end of time. Stop after a couple of minutes.
-	CMonotonicClock m_lastCommandCompletionTime;
+	fz::monotonic_clock m_lastCommandCompletionTime;
 
-	timer_id m_idleTimer{};
+	fz::timer_id m_idleTimer{};
 
 	CLatencyMeasurement m_rtt;
 
-	virtual void operator()(CEventBase const& ev);
+	virtual void operator()(fz::event_base const& ev);
 
 	void OnExternalIPAddress();
-	void OnTimer(timer_id id);
+	void OnTimer(fz::timer_id id);
 
 	wxRegEx m_pasvReplyRegex; // Have it as class member to avoid recompiling the regex on each transfer or listing
 };
@@ -173,7 +173,7 @@ public:
 	TransferEndReason transferEndReason{TransferEndReason::successful};
 	bool tranferCommandSent{};
 
-	wxLongLong resumeOffset{};
+	int64_t resumeOffset{};
 	bool binary{true};
 };
 
