@@ -1,30 +1,5 @@
 #include <filezilla.h>
 
-wxDEFINE_EVENT(fzEVT_NOTIFICATION, wxFzEvent);
-
-wxFzEvent::wxFzEvent()
-	: wxEvent(wxID_ANY, fzEVT_NOTIFICATION)
-	, engine_()
-{
-}
-
-wxFzEvent::wxFzEvent(CFileZillaEngine const* engine)
-	: wxEvent(wxID_ANY, fzEVT_NOTIFICATION)
-	, engine_(engine)
-{
-}
-
-wxFzEvent::wxFzEvent(int id)
-	: wxEvent(id, fzEVT_NOTIFICATION)
-	, engine_()
-{
-}
-
-wxEvent *wxFzEvent::Clone() const
-{
-	return new wxFzEvent(*this);
-}
-
 CDirectoryListingNotification::CDirectoryListingNotification(const CServerPath& path, const bool modified /*=false*/, const bool failed /*=false*/)
 	: m_modified(modified), m_failed(failed), m_path(path)
 {
@@ -35,8 +10,10 @@ enum RequestId CFileExistsNotification::GetRequestID() const
 	return reqId_fileexists;
 }
 
-CInteractiveLoginNotification::CInteractiveLoginNotification(const wxString& challenge)
+CInteractiveLoginNotification::CInteractiveLoginNotification(type t, const wxString& challenge, bool repeated)
 	: m_challenge(challenge)
+	, m_type(t)
+	, m_repeated(repeated)
 {
 }
 
@@ -105,7 +82,7 @@ char* CDataNotification::Detach(int& len)
 
 CCertificate::CCertificate(
 		unsigned char const* rawData, unsigned int len,
-		wxDateTime const& activationTime, wxDateTime const& expirationTime,
+		fz::datetime const& activationTime, fz::datetime const& expirationTime,
 		wxString const& serial,
 		wxString const& pkalgoname, unsigned int bits,
 		wxString const& signalgoname,
@@ -214,9 +191,11 @@ CCertificateNotification::CCertificateNotification(const wxString& host, unsigne
 		const wxString& keyExchange,
 		const wxString& sessionCipher,
 		const wxString& sessionMac,
+		int algorithmWarnings,
 		const std::vector<CCertificate> &certificates)
-	: m_protocol( protocol )
-	, m_keyExchange( keyExchange )
+	: m_protocol(protocol)
+	, m_keyExchange(keyExchange)
+	, m_algorithmWarnings(algorithmWarnings)
 {
 	m_host = host;
 	m_port = port;
