@@ -37,10 +37,14 @@ enum sftpRequestTypes
 	sftpReqPassword,
 	sftpReqHostkey,
 	sftpReqHostkeyChanged,
+	sftpReqHostkeyBetteralg,
 	sftpReqUnknown
 };
 
-class CProcess;
+namespace fz {
+class process;
+}
+
 class CSftpInputThread;
 
 class CSftpControlSocket final : public CControlSocket, public CRateLimiterObject
@@ -74,7 +78,7 @@ protected:
 	virtual int SendNextCommand();
 	virtual int ParseSubcommandResult(int prevResult);
 
-	int ProcessReply(bool successful, const wxString& reply = _T(""));
+	void ProcessReply(int result, const wxString& reply = _T(""));
 
 	int ConnectParseResponse(bool successful, const wxString& reply);
 	int ConnectSend();
@@ -84,7 +88,7 @@ protected:
 							 const CFileTransferCommand::t_transferSettings& transferSettings);
 	int FileTransferSubcommandResult(int prevResult);
 	int FileTransferSend();
-	int FileTransferParseResponse(bool successful, const wxString& reply);
+	int FileTransferParseResponse(int result, const wxString& reply);
 
 	int ListSubcommandResult(int prevResult);
 	int ListSend();
@@ -122,10 +126,10 @@ protected:
 	// see src/putty/wildcard.c
 	wxString WildcardEscape(const wxString& file);
 
-	CProcess* m_pProcess{};
+	fz::process* m_pProcess{};
 	CSftpInputThread* m_pInputThread{};
 
-	virtual void operator()(CEventBase const& ev);
+	virtual void operator()(fz::event_base const& ev);
 	void OnSftpEvent();
 	void OnTerminate();
 
