@@ -3,9 +3,15 @@
 #include "tlssocket.h"
 #include "ControlSocket.h"
 
+#include <libfilezilla/iputils.hpp>
+
 #include <gnutls/x509.h>
 
+#if FZ_USE_GNUTLS_SYSTEM_CIPHERS
+char const ciphers[] = "@SYSTEM";
+#else
 char const ciphers[] = "SECURE256:+SECURE128:-ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-ALL:-SIGN-RSA-MD5:+CTYPE-X509:-CTYPE-OPENPGP:-VERS-SSL3.0";
+#endif
 
 #define TLSDEBUG 0
 #if TLSDEBUG
@@ -21,8 +27,58 @@ void log_func(int level, const char* msg)
 }
 #endif
 
-CTlsSocket::CTlsSocket(CEventHandler* pEvtHandler, CSocket& socket, CControlSocket* pOwner)
-	: CEventHandler(pOwner->event_loop_)
+class CTlsSocketCallbacks
+{
+public:
+	static int handshake_hook_func(gnutls_session_t session, unsigned int htype, unsigned int post, unsigned int incoming)
+	{
+		if (!session) {
+			return 0;
+		}
+		auto* tls = reinterpret_cast<CTlsSocket*>(gnutls_session_get_ptr(session));
+		if (!tls) {
+			return 0;
+		}
+
+		char const* prefix;
+		if (incoming) {
+			if (post) {
+				prefix = "Processed";
+			}
+			else {
+				prefix = "Received";
+			}
+		}
+		else {
+			if (post) {
+				prefix = "Sent";
+			}
+			else {
+				prefix = "About to send";
+			}
+		}
+
+		char const* name = gnutls_handshake_description_get_name(static_cast<gnutls_handshake_description_t>(htype));
+
+		tls->m_pOwner->LogMessage(MessageType::Debug_Debug, _T("TLS handshake: %s %s"), prefix, name);
+
+		return 0;
+	}
+};
+
+namespace {
+#if GNUTLS_VERSION_NUMBER >= 0x030400
+extern "C" int handshake_hook_func(gnutls_session_t session, unsigned int htype, unsigned int post, unsigned int incoming, gnutls_datum_t const*)
+#else
+extern "C" int handshake_hook_func(gnutls_session_t session, unsigned int htype, unsigned int post, unsigned int incoming)
+#endif
+{
+	return CTlsSocketCallbacks::handshake_hook_func(session, htype, post, incoming);
+}
+}
+
+CTlsSocket::CTlsSocket(event_handler* pEvtHandler, CSocket& socket, CControlSocket* pOwner)
+	: event_handler(pOwner->event_loop_)
 	, CBackend(pEvtHandler)
 	, m_pOwner(pOwner)
 	, m_socket(socket)
@@ -35,7 +91,7 @@ CTlsSocket::CTlsSocket(CEventHandler* pEvtHandler, CSocket& socket, CControlSock
 
 CTlsSocket::~CTlsSocket()
 {
-	RemoveHandler();
+	remove_handler();
 
 	Uninit();
 	delete m_pSocketBackend;
@@ -76,8 +132,30 @@ bool CTlsSocket::Init()
 	return true;
 }
 
+bool CTlsSocket::SetClientCertificate(fz::native_string const& keyfile, fz::native_string const& certs, fz::native_string const& password)
+{
+	if (!m_certCredentials) {
+		return false;
+	}
+
+	int res = gnutls_certificate_set_x509_key_file2(m_certCredentials, fz::to_string(certs).c_str(),
+		fz::to_string(keyfile).c_str(), GNUTLS_X509_FMT_PEM, password.empty() ? 0 : fz::to_utf8(password).c_str(), 0);
+	if (res < 0) {
+		LogError(res, _T("gnutls_certificate_set_x509_key_file2"));
+		Uninit();
+		return false;
+	}
+
+	return true;
+}
+
 bool CTlsSocket::InitSession()
 {
+	if (!m_certCredentials) {
+		Uninit();
+		return false;
+	}
+
 	int res = gnutls_init(&m_session, GNUTLS_CLIENT);
 	if (res) {
 		LogError(res, _T("gnutls_init"));
@@ -85,6 +163,9 @@ bool CTlsSocket::InitSession()
 		return false;
 	}
 
+	// For use in callbacks
+	gnutls_session_set_ptr(m_session, this);
+
 	// Even though the name gnutls_db_set_cache_expiration
 	// implies expiration of some cache, it also governs
 	// the actual session lifetime, independend whether the
@@ -271,7 +352,7 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 		m_canReadFromSocket = false;
 		if (error == EAGAIN) {
 			if (m_canCheckCloseSocket && !m_pSocketBackend->IsWaiting(CRateLimiter::inbound)) {
-				SendEvent<CSocketEvent>(m_pSocketBackend, SocketEventType::close, 0);
+				send_event<CSocketEvent>(m_pSocketBackend, SocketEventType::close, 0);
 			}
 		}
 		else {
@@ -285,7 +366,7 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 	}
 
 	if (m_canCheckCloseSocket) {
-		SendEvent<CSocketEvent>(m_pSocketBackend, SocketEventType::close, 0);
+		send_event<CSocketEvent>(m_pSocketBackend, SocketEventType::close, 0);
 	}
 
 	if (!read) {
@@ -299,9 +380,9 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 	return read;
 }
 
-void CTlsSocket::operator()(CEventBase const& ev)
+void CTlsSocket::operator()(fz::event_base const& ev)
 {
-	Dispatch<CSocketEvent>(ev, this, &CTlsSocket::OnSocketEvent);
+	fz::dispatch<CSocketEvent>(ev, this, &CTlsSocket::OnSocketEvent);
 }
 
 void CTlsSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
@@ -338,7 +419,7 @@ void CTlsSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error
 			m_pOwner->LogMessage(MessageType::Debug_Info, _T("CTlsSocket::OnSocketEvent(): close event received"));
 
 			//Uninit();
-			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, 0);
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, 0);
 		}
 		break;
 	default:
@@ -430,7 +511,7 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 
 	m_tlsState = TlsState::handshake;
 
-	wxString hostname;
+	fz::native_string hostname;
 
 	if (pPrimarySocket) {
 		// Implicitly trust certificate of primary socket
@@ -454,16 +535,18 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 		hostname = m_socket.GetPeerHost();
 	}
 
-	if( !hostname.empty() && !IsIpAddress(hostname) ) {
-		const wxWX2MBbuf utf8 = hostname.mb_str(wxConvUTF8);
-		if( utf8 ) {
-			int res = gnutls_server_name_set( m_session, GNUTLS_NAME_DNS, utf8, strlen(utf8) );
-			if( res ) {
+	if (!hostname.empty() && fz::get_address_type(hostname) == fz::address_type::unknown) {
+		auto const utf8 = fz::to_utf8(hostname);
+		if (!utf8.empty()) {
+			int res = gnutls_server_name_set(m_session, GNUTLS_NAME_DNS, utf8.c_str(), utf8.size());
+			if (res) {
 				LogError(res, _T("gnutls_server_name_set"), MessageType::Debug_Warning );
 			}
 		}
 	}
 
+	gnutls_handshake_set_hook_function(m_session, GNUTLS_HANDSHAKE_ANY, GNUTLS_HOOK_BOTH, &handshake_hook_func);
+
 	return ContinueHandshake();
 }
 
@@ -500,7 +583,7 @@ int CTlsSocket::ContinueHandshake()
 		if (m_shutdown_requested) {
 			int error = Shutdown();
 			if (!error || error != EAGAIN) {
-				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, 0);
+				m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, 0);
 			}
 		}
 
@@ -642,12 +725,12 @@ void CTlsSocket::TriggerEvents()
 		return;
 
 	if (m_canTriggerRead) {
-		m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::read, 0);
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::read, 0);
 		m_canTriggerRead = false;
 	}
 
 	if (m_canTriggerWrite) {
-		m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::write, 0);
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::write, 0);
 		m_canTriggerWrite = false;
 	}
 }
@@ -719,7 +802,7 @@ void CTlsSocket::Failure(int code, bool send_close, const wxString& function)
 	Uninit();
 
 	if (send_close) {
-		m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, m_socket_error);
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, m_socket_error);
 	}
 }
 
@@ -791,7 +874,7 @@ void CTlsSocket::ContinueShutdown()
 	if (!res) {
 		m_tlsState = TlsState::closed;
 
-		m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, 0);
+		m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::close, 0);
 
 		return;
 	}
@@ -815,7 +898,7 @@ void CTlsSocket::TrustCurrentCert(bool trusted)
 		CheckResumeFailedReadWrite();
 
 		if (m_tlsState == TlsState::conn) {
-			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
+			m_pEvtHandler->send_event<CSocketEvent>(this, SocketEventType::connection, 0);
 		}
 
 		TriggerEvents();
@@ -855,8 +938,8 @@ bool CTlsSocket::ExtractCert(gnutls_datum_t const* datum, CCertificate& out)
 		return false;
 	}
 
-	CDateTime expirationTime(gnutls_x509_crt_get_expiration_time(cert), CDateTime::seconds);
-	CDateTime activationTime(gnutls_x509_crt_get_activation_time(cert), CDateTime::seconds);
+	fz::datetime expirationTime(gnutls_x509_crt_get_expiration_time(cert), fz::datetime::seconds);
+	fz::datetime activationTime(gnutls_x509_crt_get_activation_time(cert), fz::datetime::seconds);
 
 	// Get the serial number of the certificate
 	unsigned char buffer[40];
@@ -997,11 +1080,66 @@ std::vector<wxString> CTlsSocket::GetCertSubjectAltNames(gnutls_x509_crt_t cert)
 	return ret;
 }
 
-bool CTlsSocket::CertificateIsBlacklisted(std::vector<CCertificate> const& certificates)
+bool CTlsSocket::CertificateIsBlacklisted(std::vector<CCertificate> const&)
 {
 	return false;
 }
 
+
+int CTlsSocket::GetAlgorithmWarnings()
+{
+	int algorithmWarnings{};
+
+	switch (gnutls_protocol_get_version(m_session))
+	{
+		case GNUTLS_SSL3:
+		case GNUTLS_VERSION_UNKNOWN:
+			algorithmWarnings |= CCertificateNotification::tlsver;
+			break;
+		default:
+			break;
+	}
+
+	switch (gnutls_cipher_get(m_session)) {
+		case GNUTLS_CIPHER_UNKNOWN:
+		case GNUTLS_CIPHER_NULL:
+		case GNUTLS_CIPHER_ARCFOUR_128:
+		case GNUTLS_CIPHER_3DES_CBC:
+		case GNUTLS_CIPHER_ARCFOUR_40:
+		case GNUTLS_CIPHER_RC2_40_CBC:
+		case GNUTLS_CIPHER_DES_CBC:
+			algorithmWarnings |= CCertificateNotification::cipher;
+			break;
+		default:
+			break;
+	}
+
+	switch (gnutls_mac_get(m_session)) {
+		case GNUTLS_MAC_UNKNOWN:
+		case GNUTLS_MAC_NULL:
+		case GNUTLS_MAC_MD5:
+		case GNUTLS_MAC_MD2:
+		case GNUTLS_MAC_UMAC_96:
+			algorithmWarnings |= CCertificateNotification::mac;
+			break;
+		default:
+			break;
+	}
+
+	switch (gnutls_kx_get(m_session)) {
+		case GNUTLS_KX_UNKNOWN:
+		case GNUTLS_KX_ANON_DH:
+		case GNUTLS_KX_RSA_EXPORT:
+		case GNUTLS_KX_ANON_ECDH:
+			algorithmWarnings |= CCertificateNotification::kex;
+		default:
+			break;
+	}
+
+	return algorithmWarnings;
+}
+
+
 int CTlsSocket::VerifyCertificate()
 {
 	if (m_tlsState != TlsState::handshake) {
@@ -1082,6 +1220,8 @@ int CTlsSocket::VerifyCertificate()
 		return FZ_REPLY_ERROR;
 	}
 
+	int const algorithmWarnings = GetAlgorithmWarnings();
+
 	CCertificateNotification *pNotification = new CCertificateNotification(
 		m_pOwner->GetCurrentServer()->GetHost(),
 		m_pOwner->GetCurrentServer()->GetPort(),
@@ -1089,6 +1229,7 @@ int CTlsSocket::VerifyCertificate()
 		GetKeyExchange(),
 		GetCipherName(),
 		GetMacName(),
+		algorithmWarnings,
 		certificates);
 
 	m_pOwner->SendAsyncRequest(pNotification);
@@ -1096,7 +1237,7 @@ int CTlsSocket::VerifyCertificate()
 	return FZ_REPLY_WOULDBLOCK;
 }
 
-void CTlsSocket::OnRateAvailable(enum CRateLimiter::rate_direction)
+void CTlsSocket::OnRateAvailable(CRateLimiter::rate_direction)
 {
 }
 
