@@ -3,29 +3,6 @@
 
 #include "socket.h"
 
-// Also verifies that it is a correct IPv6 address
-wxString GetIPV6LongForm(wxString short_address);
-
-int DigitHexToDecNum(wxChar c);
-
-bool IsRoutableAddress(const wxString& address, CSocket::address_family family);
-
-bool IsIpAddress(const wxString& address);
-
-// Get a random number uniformly distributed in the closed interval [min, max]
-int GetRandomNumber(int low, int high);
-
-// Under some locales (e.g. Turkish), there is a different
-// relationship between the letters a-z and A-Z.
-// In Turkish for example there are different types of i
-// (dotted and dotless), with i lowercase dotted and I
-// uppercase dotless.
-// If needed, use this function to transform the case manually
-// and locale-independently
-// In addition to the usual A-Z to a-z, the other two i's are
-// transformed to lowercase dotted i as well.
-void MakeLowerAscii(wxString& str);
-
 // Strongly typed enum would be nice, but we need to support older compilers still.
 namespace dependency {
 enum type {
@@ -60,4 +37,6 @@ unique_static_cast(std::unique_ptr<Base>&& p)
 #define wxPLURAL_LL(sing, plur, n) \
 	wxGetTranslation((sing), (plur), (sizeof(unsigned int) < 8 && (n) > 1000000000) ? (1000000000 + (n) % 1000000000) : (n))
 
+std::wstring url_encode(std::wstring const& s, bool keep_slashes = false);
+
 #endif //__MISC_H__
