@@ -1,16 +1,16 @@
 #ifndef __SERVERPATH_H__
 #define __SERVERPATH_H__
 
-#include "optional.h"
-#include "refcount.h"
+#include <libfilezilla/optional.hpp>
+#include <libfilezilla/shared.hpp>
 
 #include <deque>
 
 class CServerPathData final
 {
 public:
-	std::deque<wxString> m_segments;
-	CSparseOptional<wxString> m_prefix;
+	std::deque<std::wstring> m_segments;
+	fz::sparse_optional<std::wstring> m_prefix;
 
 	bool operator==(const CServerPathData& cmp) const;
 };
@@ -20,8 +20,12 @@ class CServerPath final
 public:
 	CServerPath();
 	explicit CServerPath(wxString const& path, ServerType type = DEFAULT);
-	CServerPath(const CServerPath &path, wxString subdir); // Ignores parent on absolute subdir
-	CServerPath(const CServerPath &path);
+	CServerPath(CServerPath const& path, wxString subdir); // Ignores parent on absolute subdir
+	CServerPath(CServerPath const& path) = default;
+	CServerPath(CServerPath && path) noexcept = default;
+
+	CServerPath& operator=(CServerPath const& op) = default;
+	CServerPath& operator=(CServerPath && op) noexcept = default;
 
 	bool empty() const { return !m_data; }
 	void clear();
@@ -32,7 +36,7 @@ public:
 
 	// If ChangePath returns false, the object will be left
 	// empty.
-	bool ChangePath(wxString subdir);
+	bool ChangePath(wxString const& subdir);
 	bool ChangePath(wxString &subdir, bool isFile);
 
 	wxString GetPath() const;
@@ -65,22 +69,24 @@ public:
 
 	bool AddSegment(const wxString& segment);
 
-protected:
+	size_t SegmentCount() const;
+private:
 	bool DoSetSafePath(const wxString& path);
 	bool DoChangePath(wxString &subdir, bool isFile);
 
 	ServerType m_type;
 
-	typedef std::deque<wxString> tSegmentList;
+	typedef std::deque<std::wstring> tSegmentList;
 	typedef tSegmentList::iterator tSegmentIter;
 	typedef tSegmentList::const_iterator tConstSegmentIter;
 
-	bool Segmentize(wxString str, tSegmentList& segments);
+	bool Segmentize(std::wstring const& str, tSegmentList& segments);
+	bool SegmentizeAddSegment(std::wstring & segment, tSegmentList& segments, bool& append);
 	bool ExtractFile(wxString& dir, wxString& file);
 
 	static void EscapeSeparators(ServerType type, wxString& subdir);
 
-	CRefcountObject_Uninitialized<CServerPathData> m_data;
+	fz::shared_optional<CServerPathData> m_data;
 };
 
 #endif
