@@ -280,8 +280,6 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQu
 	AddColumn(_("Last modified"), wxLIST_FORMAT_LEFT, widths[3]);
 	LoadColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
 
-	InitSort(OPTION_LOCALFILELIST_SORTORDER);
-
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
 #ifdef __WXMSW__
@@ -290,6 +288,8 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQu
 
 	InitHeaderSortImageList();
 
+	InitSort(OPTION_LOCALFILELIST_SORTORDER);
+
 	SetDropTarget(new CLocalListViewDropTarget(this));
 
 	EnablePrefixSearch(true);
@@ -359,11 +359,11 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 	if (m_dir.GetPath() == _T("\\")) {
 		DisplayDrives();
 	}
-	else if (m_dir.GetPath().Left(2) == _T("\\\\"))
-	{
-		int pos = m_dir.GetPath().Mid(2).Find('\\');
-		if (pos != -1 && pos + 3 != (int)m_dir.GetPath().Len())
+	else if (m_dir.GetPath().substr(0, 2) == _T("\\\\")) {
+		auto pos = m_dir.GetPath().substr(2).find('\\');
+		if (pos != std::wstring::npos && pos + 3 < m_dir.GetPath().size()) {
 			goto regular_dir;
+		}
 
 		// UNC path without shares
 		DisplayShares(m_dir.GetPath());
@@ -1273,6 +1273,10 @@ void CLocalListView::OnStateChange(t_statechange_notifications notification, con
 
 void CLocalListView::OnBeginDrag(wxListEvent&)
 {
+	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+		return;
+	}
+
 	long item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
@@ -1488,8 +1492,7 @@ wxListItemAttr* CLocalListView::OnGetItemAttr(long item) const
 
 void CLocalListView::StartComparison()
 {
-	if (m_sortDirection || m_sortColumn)
-	{
+	if (m_sortDirection || m_sortColumn != 0) {
 		wxASSERT(m_originalIndexMapping.empty());
 		SortList(0, 0);
 	}
@@ -1581,7 +1584,7 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 			return wxString();
 
 		if (data->fileType.empty())
-			data->fileType = GetType(data->name, data->dir, m_dir.GetPath());
+			data->fileType = GetType(data->name, data->dir, m_dir.GetPath()).ToStdWstring();
 
 		return data->fileType;
 	}
