@@ -3,6 +3,7 @@
 
 #include "statusbar.h"
 #include "engine_context.h"
+#include "notification.h"
 
 #include <wx/timer.h>
 
@@ -30,7 +31,7 @@ class CThemeProvider;
 class CToolBar;
 class CWindowStateManager;
 
-class CMainFrame final : public wxNavigationEnabled<wxFrame>
+class CMainFrame final : public wxNavigationEnabled<wxFrame>, private COptionChangeEventHandler, public EngineNotificationHandler
 #if FZ_MANUALUPDATECHECK
 	, protected CUpdateHandler
 #endif
@@ -46,8 +47,6 @@ public:
 	CQueueView* GetQueue() { return m_pQueueView; }
 	CQuickconnectBar* GetQuickconnectBar() { return m_pQuickconnectBar; }
 
-	void UpdateLayout(int layout = -1, int swap = -1, int messagelog_position = -1);
-
 	// Window size and position as well as pane sizes
 	void RememberSplitterPositions();
 	bool RestoreSplitterPositions();
@@ -70,7 +69,8 @@ public:
 	bool ConnectToSite(CSiteManagerItemData_Site & data, bool newTab = false);
 
 	CFileZillaEngineContext& GetEngineContext() { return m_engineContext; }
-protected:
+private:
+	void UpdateLayout();
 	void FixTabOrder();
 
 	bool CloseDialogsAndQuit(wxCloseEvent &event);
@@ -118,11 +118,15 @@ protected:
 
 	void SetupKeyboardAccelerators();
 
+	void OnOptionsChanged(changed_options_t const& options);
+
+	virtual void OnEngineEvent(CFileZillaEngine* engine); // Before thread jump
+	void DoOnEngineEvent(CFileZillaEngine* engine); // After thread jump
+
 	// Event handlers
 	DECLARE_EVENT_TABLE()
 	void OnSize(wxSizeEvent& event);
 	void OnMenuHandler(wxCommandEvent& event);
-	void OnEngineEvent(wxFzEvent& event);
 	void OnUpdateLedTooltip(wxCommandEvent&);
 	void OnDisconnect(wxCommandEvent&);
 	void OnCancel(wxCommandEvent&);
