@@ -7,10 +7,10 @@
 #include "xmlfunctions.h"
 #include "filezillaapp.h"
 #include "ipcmutex.h"
+#include "local_recursive_operation.h"
 #include "state.h"
 #include "asyncrequestqueue.h"
 #include "defaultfileexistsdlg.h"
-#include "filter.h"
 #include <wx/dnd.h>
 #include "dndobjects.h"
 #include "loginmanager.h"
@@ -22,7 +22,7 @@
 #include <wx/progdlg.h>
 #include <wx/sound.h>
 #include "statusbar.h"
-#include "recursive_operation.h"
+#include "remote_recursive_operation.h"
 #include "auto_ascii_files.h"
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
@@ -32,8 +32,6 @@
 #include <wx/notifmsg.h>
 #endif
 
-#include <libfilezilla/local_filesys.hpp>
-
 #ifdef __WXMSW__
 #include <powrprof.h>
 #endif
@@ -68,8 +66,7 @@ public:
 		if (pDragDropManager)
 			pDragDropManager->pDropTarget = m_pQueueView;
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
-		{
+		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
 			CState* const pState = CContextManager::Get()->GetCurrentContext();
 			if (!pState)
 				return wxDragNone;
@@ -83,10 +80,8 @@ public:
 
 			pState->UploadDroppedFiles(m_pFileDataObject, path, true);
 		}
-		else
-		{
-			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
-			{
+		else {
+			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
@@ -98,15 +93,13 @@ public:
 			if (!pServer)
 				return wxDragNone;
 
-			if (!pServer->EqualsNoPass(m_pRemoteDataObject->GetServer()))
-			{
+			if (!pServer->EqualsNoPass(m_pRemoteDataObject->GetServer())) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
 			const CLocalPath& target = pState->GetLocalDir();
-			if (!target.IsWriteable())
-			{
+			if (!target.IsWriteable()) {
 				wxBell();
 				return wxDragNone;
 			}
@@ -167,20 +160,7 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-DECLARE_EVENT_TYPE(fzEVT_FOLDERTHREAD_COMPLETE, -1)
-DEFINE_EVENT_TYPE(fzEVT_FOLDERTHREAD_COMPLETE)
-
-DECLARE_EVENT_TYPE(fzEVT_FOLDERTHREAD_FILES, -1)
-DEFINE_EVENT_TYPE(fzEVT_FOLDERTHREAD_FILES)
-
-DECLARE_EVENT_TYPE(fzEVT_ASKFORPASSWORD, -1)
-DEFINE_EVENT_TYPE(fzEVT_ASKFORPASSWORD)
-
 BEGIN_EVENT_TABLE(CQueueView, CQueueViewBase)
-EVT_FZ_NOTIFICATION(wxID_ANY, CQueueView::OnEngineEvent)
-EVT_COMMAND(wxID_ANY, fzEVT_FOLDERTHREAD_COMPLETE, CQueueView::OnFolderThreadComplete)
-EVT_COMMAND(wxID_ANY, fzEVT_FOLDERTHREAD_FILES, CQueueView::OnFolderThreadFiles)
-
 EVT_CONTEXT_MENU(CQueueView::OnContextMenu)
 EVT_MENU(XRCID("ID_PROCESSQUEUE"), CQueueView::OnProcessQueue)
 EVT_MENU(XRCID("ID_REMOVEALL"), CQueueView::OnStopAndClear)
@@ -197,8 +177,6 @@ EVT_MENU(XRCID("ID_ACTIONAFTER_REBOOT"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_SHUTDOWN"), CQueueView::OnActionAfter)
 EVT_MENU(XRCID("ID_ACTIONAFTER_SLEEP"), CQueueView::OnActionAfter)
 
-EVT_COMMAND(wxID_ANY, fzEVT_ASKFORPASSWORD, CQueueView::OnAskPassword)
-
 EVT_TIMER(wxID_ANY, CQueueView::OnTimer)
 EVT_CHAR(CQueueView::OnChar)
 
@@ -213,230 +191,6 @@ EVT_COMMAND(wxID_ANY, fzEVT_GRANTEXCLUSIVEENGINEACCESS, CQueueView::OnExclusiveE
 EVT_SIZE(CQueueView::OnSize)
 END_EVENT_TABLE()
 
-class CFolderProcessingThread final : public fz::thread
-{
-	struct t_internalDirPair
-	{
-		CLocalPath localPath;
-		CServerPath remotePath;
-	};
-public:
-	CFolderProcessingThread(CQueueView* pOwner, CFolderScanItem* pFolderItem)
-	{
-		m_pOwner = pOwner;
-		m_pFolderItem = pFolderItem;
-
-		t_internalDirPair* pair = new t_internalDirPair;
-		pair->localPath = pFolderItem->GetLocalPath();
-		pair->remotePath = pFolderItem->GetRemotePath();
-		m_dirsToCheck.push_back(pair);
-	}
-
-	virtual ~CFolderProcessingThread()
-	{
-		{
-			fz::scoped_lock locker(m_sync);
-			m_quit = true;
-		}
-		join();
-		for (auto iter = m_entryList.begin(); iter != m_entryList.end(); ++iter)
-			delete *iter;
-		for (auto iter = m_dirsToCheck.begin(); iter != m_dirsToCheck.end(); ++iter)
-			delete *iter;
-	}
-
-	void GetFiles(std::list<CFolderProcessingEntry*> &entryList)
-	{
-		wxASSERT(entryList.empty());
-		fz::scoped_lock locker(m_sync);
-		entryList.swap(m_entryList);
-
-		m_didSendEvent = false;
-		m_processing_entries = true;
-
-		if (m_throttleWait) {
-			m_throttleWait = false;
-			m_condition.signal(locker);
-		}
-	}
-
-	class t_dirPair : public CFolderProcessingEntry
-	{
-	public:
-		t_dirPair() : CFolderProcessingEntry(CFolderProcessingEntry::dir) {}
-		CLocalPath localPath;
-		CServerPath remotePath;
-	};
-
-	void ProcessDirectory(const CLocalPath& localPath, CServerPath const& remotePath, const wxString& name)
-	{
-		fz::scoped_lock locker(m_sync);
-
-		t_internalDirPair* pair = new t_internalDirPair;
-
-		{
-			pair->localPath = localPath;
-			pair->localPath.AddSegment(name);
-
-			pair->remotePath = remotePath;
-			pair->remotePath.AddSegment(name);
-		}
-
-		m_dirsToCheck.push_back(pair);
-
-		if (m_threadWaiting) {
-			m_threadWaiting = false;
-			m_condition.signal(locker);
-		}
-	}
-
-	void CheckFinished()
-	{
-		fz::scoped_lock locker(m_sync);
-		wxASSERT(m_processing_entries);
-
-		m_processing_entries = false;
-
-		if (m_threadWaiting && (!m_dirsToCheck.empty() || m_entryList.empty())) {
-			m_threadWaiting = false;
-			m_condition.signal(locker);
-		}
-	}
-
-	CFolderScanItem* GetFolderScanItem()
-	{
-		return m_pFolderItem;
-	}
-
-protected:
-
-	void AddEntry(CFolderProcessingEntry* entry)
-	{
-		fz::scoped_lock l(m_sync);
-		m_entryList.push_back(entry);
-
-		// Wait if there are more than 100 items to queue,
-		// don't send notification if there are less than 10.
-		// This reduces overhead
-		bool send;
-
-		if (m_didSendEvent) {
-			send = false;
-			if (m_entryList.size() >= 100) {
-				m_throttleWait = true;
-				m_condition.wait(l);
-			}
-		}
-		else if (m_entryList.size() < 20)
-			send = false;
-		else
-			send = true;
-
-		if (send)
-			m_didSendEvent = true;
-
-		l.unlock();
-
-		if (send) {
-			// We send the notification after leaving the critical section, else we
-			// could get into a deadlock. wxWidgets event system does internal
-			// locking.
-			m_pOwner->QueueEvent(new wxCommandEvent(fzEVT_FOLDERTHREAD_FILES, wxID_ANY));
-		}
-	}
-
-	void entry()
-	{
-#ifdef __WXDEBUG__
-		wxMutexGuiEnter();
-		wxASSERT(m_pFolderItem->GetTopLevelItem() && m_pFolderItem->GetTopLevelItem()->GetType() == QueueItemType::Server);
-		wxMutexGuiLeave();
-#endif
-
-		wxASSERT(!m_pFolderItem->Download());
-
-		fz::local_filesys localFileSystem;
-
-		while (!m_pFolderItem->m_remove) {
-			fz::scoped_lock l(m_sync);
-			if (m_quit) {
-				break;
-			}
-			if (m_dirsToCheck.empty()) {
-				if (!m_didSendEvent && !m_entryList.empty()) {
-					m_didSendEvent = true;
-					l.unlock();
-					m_pOwner->QueueEvent(new wxCommandEvent(fzEVT_FOLDERTHREAD_FILES, wxID_ANY));
-					continue;
-				}
-
-				if (!m_didSendEvent && !m_processing_entries) {
-					break;
-				}
-				m_threadWaiting = true;
-				m_condition.wait(l);
-				if (m_dirsToCheck.empty()) {
-					break;
-				}
-				continue;
-			}
-
-			const t_internalDirPair *pair = m_dirsToCheck.front();
-			m_dirsToCheck.pop_front();
-
-			l.unlock();
-
-			if (!localFileSystem.begin_find_files(fz::to_native(pair->localPath.GetPath()), false)) {
-				delete pair;
-				continue;
-			}
-
-			t_dirPair* pair2 = new t_dirPair;
-			pair2->localPath = pair->localPath;
-			pair2->remotePath = pair->remotePath;
-			AddEntry(pair2);
-
-			t_newEntry* entry = new t_newEntry;
-
-			fz::native_string name;
-			bool is_link;
-			bool is_dir;
-			while (localFileSystem.get_next_file(name, is_link, is_dir, &entry->size, &entry->time, &entry->attributes)) {
-				if (is_link)
-					continue;
-
-				entry->name = name;
-				entry->dir = is_dir;
-
-				AddEntry(entry);
-
-				entry = new t_newEntry;
-			}
-			delete entry;
-
-			delete pair;
-		}
-
-		m_pOwner->QueueEvent(new wxCommandEvent(fzEVT_FOLDERTHREAD_COMPLETE, wxID_ANY));
-	}
-
-	std::list<t_internalDirPair*> m_dirsToCheck;
-
-	// Access has to be guarded by m_sync
-	std::list<CFolderProcessingEntry*> m_entryList;
-
-	CQueueView* m_pOwner;
-	CFolderScanItem* m_pFolderItem;
-
-	fz::mutex m_sync;
-	fz::condition m_condition;
-	bool m_threadWaiting{};
-	bool m_throttleWait{};
-	bool m_didSendEvent{};
-	bool m_processing_entries{};
-	bool m_quit{};
-};
-
 CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsyncRequestQueue *pAsyncRequestQueue)
 	: CQueueViewBase(parent, index, _("Queued files")),
 	m_pMainFrame(pMainFrame),
@@ -464,8 +218,6 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 
 	SetDropTarget(new CQueueViewDropTarget(this));
 
-	m_folderscan_item_refresh_timer.SetOwner(this);
-
 	m_line_height = -1;
 #ifdef __WXMSW__
 	m_header_height = -1;
@@ -476,9 +228,6 @@ CQueueView::CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsync
 
 CQueueView::~CQueueView()
 {
-	delete m_pFolderProcessingThread;
-	m_pFolderProcessingThread = 0;
-
 	DeleteEngines();
 
 	m_resize_timer.Stop();
@@ -523,18 +272,16 @@ bool CQueueView::QueueFile(const bool queueOnly, const bool download,
 void CQueueView::QueueFile_Finish(const bool start)
 {
 	bool need_refresh = false;
-	if (m_insertionStart <= GetTopItem() + GetCountPerPage() + 1)
+	if (m_insertionStart >= 0 && m_insertionStart <= GetTopItem() + GetCountPerPage() + 1)
 		need_refresh = true;
 	CommitChanges();
 
-	if (!m_activeMode && start)
-	{
+	if (!m_activeMode && start) {
 		m_activeMode = 1;
 		CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUEUEPROCESSING);
 	}
 
-	if (m_activeMode)
-	{
+	if (m_activeMode) {
 		m_waitStatusLineUpdate = true;
 		AdvanceQueue(false);
 		m_waitStatusLineUpdate = false;
@@ -559,12 +306,11 @@ bool CQueueView::QueueFiles(const bool queueOnly, const CLocalPath& localPath, c
 		if (fileInfo.dir)
 			continue;
 
-		CFileItem* fileItem;
 		wxString localFile = ReplaceInvalidCharacters(fileInfo.name);
 		if (dataObject.GetServerPath().GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
 			localFile = StripVMSRevision(localFile);
 
-		fileItem = new CFileItem(pServerItem, queueOnly, true,
+		CFileItem* fileItem = new CFileItem(pServerItem, queueOnly, true,
 			fileInfo.name, (fileInfo.name != localFile) ? localFile : wxString(),
 			localPath, dataObject.GetServerPath(), fileInfo.size);
 		fileItem->SetAscii(CAutoAsciiFiles::TransferRemoteAsAscii(fileInfo.name, dataObject.GetServerPath().GetType()));
@@ -577,9 +323,43 @@ bool CQueueView::QueueFiles(const bool queueOnly, const CLocalPath& localPath, c
 	return true;
 }
 
-void CQueueView::OnEngineEvent(wxFzEvent &event)
+bool CQueueView::QueueFiles(const bool queueOnly, CServer const& server, CLocalRecursiveOperation::listing const& listing)
 {
-	t_EngineData* const pEngineData = GetEngineData(event.engine_);
+	CServerItem* pServerItem = CreateServerItem(server);
+
+	auto const& files = listing.files;
+	if (files.empty() && listing.dirs.empty()) {
+		// Empty directory
+		if (listing.remotePath.HasParent()) {
+			wxString sub = listing.remotePath.GetLastSegment();
+		}
+		CFileItem* fileItem = new CFolderItem(pServerItem, queueOnly, listing.remotePath, wxString());
+		InsertItem(pServerItem, fileItem);
+	}
+	else {
+		for (auto const& file : files) {
+			CFileItem* fileItem = new CFileItem(pServerItem, queueOnly, false,
+				file.name, wxString(),
+				listing.localPath, listing.remotePath, file.size);
+			fileItem->SetAscii(CAutoAsciiFiles::TransferLocalAsAscii(file.name, listing.remotePath.GetType()));
+
+			InsertItem(pServerItem, fileItem);
+		}
+
+		// We do not look at dirs here, recursion takes care of it.
+	}
+	
+	return true;
+}
+
+void CQueueView::OnEngineEvent(CFileZillaEngine* engine)
+{
+	CallAfter(&CQueueView::DoOnEngineEvent, engine);
+}
+
+void CQueueView::DoOnEngineEvent(CFileZillaEngine* engine)
+{
+	t_EngineData* const pEngineData = GetEngineData(engine);
 	if (!pEngineData)
 		return;
 
@@ -1331,7 +1111,7 @@ void CQueueView::SendNextCommand(t_EngineData& engineData)
 			engineData.pItem->SetStatusMessage(CFileItem::wait_password);
 			RefreshItem(engineData.pItem);
 			if (m_waitingForPassword.empty()) {
-				QueueEvent(new wxCommandEvent(fzEVT_ASKFORPASSWORD));
+				CallAfter(&CQueueView::OnAskPassword);
 			}
 			m_waitingForPassword.push_back(engineData.pEngine);
 			return;
@@ -1436,7 +1216,7 @@ void CQueueView::SendNextCommand(t_EngineData& engineData)
 	}
 }
 
-bool CQueueView::SetActive(bool active /*=true*/)
+bool CQueueView::SetActive(bool active)
 {
 	if (!active) {
 		m_activeMode = 0;
@@ -1447,14 +1227,20 @@ bool CQueueView::SetActive(bool active /*=true*/)
 		for (std::vector<CState*>::const_iterator iter = pStates->begin(); iter != pStates->end(); ++iter) {
 			CState* pState = *iter;
 
-			CRecursiveOperation* pRecursiveOperation = pState->GetRecursiveOperationHandler();
-			if (!pRecursiveOperation)
-				continue;
-
-			if (pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download)
-				pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue);
-			if (pRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_download_flatten)
-				pRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue_flatten);
+			CLocalRecursiveOperation* pLocalRecursiveOperation = pState->GetLocalRecursiveOperation();
+			if (pLocalRecursiveOperation) {
+				if (pLocalRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_transfer)
+					pLocalRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue);
+				if (pLocalRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_transfer_flatten)
+					pLocalRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue_flatten);
+			}
+			CRemoteRecursiveOperation* pRemoteRecursiveOperation = pState->GetRemoteRecursiveOperation();
+			if (pRemoteRecursiveOperation) {
+				if (pRemoteRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_transfer)
+					pRemoteRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue);
+				if (pRemoteRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_transfer_flatten)
+					pRemoteRecursiveOperation->ChangeOperationMode(CRecursiveOperation::recursive_addtoqueue_flatten);
+			}
 		}
 
 		UpdateStatusLinePositions();
@@ -1483,12 +1269,14 @@ bool CQueueView::SetActive(bool active /*=true*/)
 		return m_activeCount == 0;
 	}
 	else {
-		m_activeMode = 2;
+		if (!m_serverList.empty()) {
+			m_activeMode = 2;
 
-		m_waitStatusLineUpdate = true;
-		AdvanceQueue();
-		m_waitStatusLineUpdate = false;
-		UpdateStatusLinePositions();
+			m_waitStatusLineUpdate = true;
+			AdvanceQueue();
+			m_waitStatusLineUpdate = false;
+			UpdateStatusLinePositions();
+		}
 	}
 
 	CContextManager::Get()->NotifyGlobalHandlers(STATECHANGE_QUEUEPROCESSING);
@@ -1514,17 +1302,6 @@ bool CQueueView::Quit()
 	if (!SetActive(false))
 		canQuit = false;
 
-	for (unsigned int i = 0; i < 2; ++i) {
-		if (!m_queuedFolders[i].empty()) {
-			canQuit = false;
-			for (auto & dir : m_queuedFolders[i]) {
-				dir->m_remove = true;
-			}
-		}
-	}
-	if (m_pFolderProcessingThread)
-		canQuit = false;
-
 	if (!canQuit)
 		return false;
 
@@ -1606,8 +1383,7 @@ void CQueueView::UpdateStatusLinePositions()
 	m_lastTopItem = GetTopItem();
 	int bottomItem = m_lastTopItem + GetCountPerPage();
 
-	for (auto iter = m_statusLineList.begin(); iter != m_statusLineList.end(); ++iter) {
-		CStatusLineCtrl *pCtrl = *iter;
+	for (auto pCtrl : m_statusLineList) {
 		int index = GetItemIndex(pCtrl->GetItem()) + 1;
 		if (index < m_lastTopItem || index > bottomItem) {
 			pCtrl->Show(false);
@@ -1634,11 +1410,10 @@ void CQueueView::CalculateQueueSize()
 	// Collect total queue size
 	m_totalQueueSize = 0;
 	m_fileCount = 0;
-	m_folderScanCount = 0;
 
 	m_filesWithUnknownSize = 0;
 	for (std::vector<CServerItem*>::const_iterator iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-		m_totalQueueSize += (*iter)->GetTotalSize(m_filesWithUnknownSize, m_fileCount, m_folderScanCount);
+		m_totalQueueSize += (*iter)->GetTotalSize(m_filesWithUnknownSize, m_fileCount);
 
 	DisplayQueueSize();
 	DisplayNumberQueuedFiles();
@@ -1652,148 +1427,6 @@ void CQueueView::DisplayQueueSize()
 	pStatusBar->DisplayQueueSize(m_totalQueueSize, m_filesWithUnknownSize != 0);
 }
 
-bool CQueueView::QueueFolder(bool queueOnly, bool download, const CLocalPath& localPath, const CServerPath& remotePath, const CServer& server)
-{
-	CServerItem* pServerItem = CreateServerItem(server);
-
-	CFolderScanItem* folderItem = new CFolderScanItem(pServerItem, queueOnly, download, localPath, remotePath);
-	InsertItem(pServerItem, folderItem);
-
-	folderItem->m_statusMessage = _("Waiting");
-
-	CommitChanges();
-
-	m_queuedFolders[download ? 0 : 1].push_back(folderItem);
-	ProcessFolderItems();
-
-	RefreshListOnly(false);
-
-	return true;
-}
-
-bool CQueueView::ProcessFolderItems(int type /*=-1*/)
-{
-	if (type == -1) {
-		while (ProcessFolderItems(0));
-		ProcessUploadFolderItems();
-
-		return true;
-	}
-
-	return false;
-}
-
-void CQueueView::ProcessUploadFolderItems()
-{
-	if (m_queuedFolders[1].empty()) {
-		if (m_quit)
-			m_pMainFrame->Close();
-
-		return;
-	}
-
-	if (m_pFolderProcessingThread)
-		return;
-
-	CFolderScanItem* pItem = m_queuedFolders[1].front();
-
-	if (pItem->queued())
-		pItem->m_statusMessage = _("Scanning for files to add to queue");
-	else
-		pItem->m_statusMessage = _("Scanning for files to upload");
-	RefreshItem(pItem);
-	pItem->m_active = true;
-	m_pFolderProcessingThread = new CFolderProcessingThread(this, pItem);
-	m_pFolderProcessingThread->run();
-
-	RefreshListOnly(false);
-}
-
-void CQueueView::OnFolderThreadComplete(wxCommandEvent&)
-{
-	if (!m_pFolderProcessingThread)
-		return;
-
-	m_folderscan_item_refresh_timer.Stop();
-
-	wxASSERT(!m_queuedFolders[1].empty());
-	CFolderScanItem* pItem = m_queuedFolders[1].front();
-	if (pItem->m_dir_is_empty) {
-		CServerItem* pServerItem = (CServerItem*)pItem->GetTopLevelItem();
-		CFileItem* fileItem = new CFolderItem(pServerItem, pItem->queued(), pItem->m_current_remote_path, _T(""));
-		InsertItem(pServerItem, fileItem);
-		QueueFile_Finish(!pItem->queued());
-	}
-	m_queuedFolders[1].pop_front();
-
-	RemoveItem(pItem, true);
-
-	delete m_pFolderProcessingThread;
-	m_pFolderProcessingThread = 0;
-
-	ProcessUploadFolderItems();
-}
-
-int CQueueView::QueueFiles(const std::list<CFolderProcessingEntry*> &entryList, bool queueOnly, bool download, CServerItem* pServerItem, const CFileExistsNotification::OverwriteAction defaultFileExistsAction)
-{
-	wxASSERT(pServerItem);
-
-	CFolderScanItem* pFolderScanItem = m_pFolderProcessingThread->GetFolderScanItem();
-
-	int added = 0;
-
-	CFilterManager filters;
-	for (std::list<CFolderProcessingEntry*>::const_iterator iter = entryList.begin(); iter != entryList.end(); ++iter) {
-		if ((*iter)->m_type == CFolderProcessingEntry::dir) {
-			if (m_pFolderProcessingThread->GetFolderScanItem()->m_dir_is_empty) {
-				CFileItem* fileItem = new CFolderItem(pServerItem, queueOnly, pFolderScanItem->m_current_remote_path, _T(""));
-				InsertItem(pServerItem, fileItem);
-				added++;
-			}
-
-			const CFolderProcessingThread::t_dirPair* entry = (const CFolderProcessingThread::t_dirPair*)*iter;
-			pFolderScanItem->m_current_local_path = entry->localPath;
-			pFolderScanItem->m_current_remote_path = entry->remotePath;
-			pFolderScanItem->m_dir_is_empty = true;
-			delete entry;
-		}
-		else {
-			const t_newEntry* entry = (const t_newEntry*)*iter;
-			if (filters.FilenameFiltered(entry->name, pFolderScanItem->m_current_local_path.GetPath(), entry->dir, entry->size, true, entry->attributes, entry->time)) {
-				delete entry;
-				continue;
-			}
-
-			pFolderScanItem->m_dir_is_empty = false;
-
-			if (entry->dir) {
-				m_pFolderProcessingThread->ProcessDirectory(pFolderScanItem->m_current_local_path, pFolderScanItem->m_current_remote_path, entry->name);
-				delete entry;
-				continue;
-			}
-
-			CFileItem* fileItem = new CFileItem(pServerItem, queueOnly, download, entry->name, wxEmptyString, pFolderScanItem->m_current_local_path, pFolderScanItem->m_current_remote_path, entry->size);
-
-			if (download)
-				fileItem->SetAscii(CAutoAsciiFiles::TransferRemoteAsAscii(entry->name, pFolderScanItem->m_current_remote_path.GetType()));
-			else
-				fileItem->SetAscii(CAutoAsciiFiles::TransferLocalAsAscii(entry->name, pFolderScanItem->m_current_remote_path.GetType()));
-
-			fileItem->m_defaultFileExistsAction = defaultFileExistsAction;
-
-			delete entry;
-
-			InsertItem(pServerItem, fileItem);
-
-			added++;
-		}
-	}
-
-	QueueFile_Finish(!queueOnly);
-
-	return added;
-}
-
 void CQueueView::SaveQueue()
 {
 	// Kiosk mode 2 doesn't save queue
@@ -1804,8 +1437,7 @@ void CQueueView::SaveQueue()
 	// just as extra precaution. Better 'save' than sorry.
 	CInterProcessMutex mutex(MUTEX_QUEUE);
 
-	if (!m_queue_storage.SaveQueue(m_serverList))
-	{
+	if (!m_queue_storage.SaveQueue(m_serverList)) {
 		wxString msg = wxString::Format(_("An error occurred saving the transfer queue to \"%s\".\nSome queue items might not have been saved."), m_queue_storage.GetDatabaseFilename());
 		wxMessageBoxEx(msg, _("Error saving queue"), wxICON_ERROR);
 	}
@@ -2150,15 +1782,6 @@ void CQueueView::RemoveAll()
 		}
 	}
 
-	// Clear list of queued directories that aren't busy
-	for (unsigned int i = 0; i < 2; ++i) {
-		auto begin = m_queuedFolders[i].begin();
-		auto end = m_queuedFolders[i].end();
-		if (begin != end && (*begin)->m_active)
-			++begin;
-		m_queuedFolders[i].erase(begin, end);
-	}
-
 	SaveSetItemCount(m_itemCount);
 
 	if (newServerList.empty() && (m_actionAfterState == ActionAfterState::Reboot || m_actionAfterState == ActionAfterState::Shutdown || m_actionAfterState == ActionAfterState::Sleep)) {
@@ -2174,19 +1797,6 @@ void CQueueView::RemoveAll()
 	RefreshListOnly();
 }
 
-void CQueueView::RemoveQueuedFolderItem(CFolderScanItem* pFolder)
-{
-	for (unsigned int i = 0; i < 2; ++i) {
-		for (auto iter = m_queuedFolders[i].begin(); iter != m_queuedFolders[i].end(); ++iter) {
-			if (*iter != pFolder)
-				continue;
-
-			m_queuedFolders[i].erase(iter);
-			return;
-		}
-	}
-}
-
 void CQueueView::OnRemoveSelected(wxCommandEvent&)
 {
 #ifndef __WXMSW__
@@ -2230,15 +1840,6 @@ void CQueueView::OnRemoveSelected(wxCommandEvent&)
 
 		if (pItem->GetType() == QueueItemType::Status)
 			continue;
-		else if (pItem->GetType() == QueueItemType::FolderScan) {
-			CFolderScanItem* pFolder = (CFolderScanItem*)pItem;
-			if (pFolder->m_active) {
-				pFolder->m_remove = true;
-				continue;
-			}
-			else
-				RemoveQueuedFolderItem(pFolder);
-		}
 		else if (pItem->GetType() == QueueItemType::Server) {
 			CServerItem* pServer = (CServerItem*)pItem;
 			StopItem(pServer, false);
@@ -2315,15 +1916,8 @@ bool CQueueView::StopItem(CServerItem* pServerItem, bool updateSelections)
 
 	for (int i = static_cast<int>(items.size()) - 1; i >= removedAtFront; --i) {
 		CQueueItem* pItem = items[i];
-		if (pItem->GetType() == QueueItemType::FolderScan) {
-			CFolderScanItem* pFolder = (CFolderScanItem*)pItem;
-			if (pFolder->m_active) {
-				pFolder->m_remove = true;
-				continue;
-			}
-		}
-		else if (pItem->GetType() == QueueItemType::File ||
-				 pItem->GetType() == QueueItemType::Folder)
+		if (pItem->GetType() == QueueItemType::File ||
+			 pItem->GetType() == QueueItemType::Folder)
 		{
 			CFileItem* pFile = (CFileItem*)pItem;
 			if (pFile->IsActive()) {
@@ -2349,25 +1943,6 @@ bool CQueueView::StopItem(CServerItem* pServerItem, bool updateSelections)
 	return false;
 }
 
-void CQueueView::OnFolderThreadFiles(wxCommandEvent&)
-{
-	if (!m_pFolderProcessingThread)
-		return;
-
-	wxASSERT(!m_queuedFolders[1].empty());
-	CFolderScanItem* pItem = m_queuedFolders[1].front();
-
-	std::list<CFolderProcessingEntry*> entryList;
-	m_pFolderProcessingThread->GetFiles(entryList);
-	int added = QueueFiles(entryList, pItem->queued(), false, (CServerItem*)pItem->GetTopLevelItem(), pItem->m_defaultFileExistsAction);
-	m_pFolderProcessingThread->CheckFinished();
-
-	pItem->m_count += added;
-
-	if (!m_folderscan_item_refresh_timer.IsRunning())
-		m_folderscan_item_refresh_timer.Start(200, true);
-}
-
 void CQueueView::SetDefaultFileExistsAction(CFileExistsNotification::OverwriteAction action, const TransferDirection direction)
 {
 	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
@@ -2404,13 +1979,6 @@ void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &)
 
 		switch (pItem->GetType())
 		{
-		case QueueItemType::FolderScan:
-			if (uploadAction == CFileExistsNotification::unknown)
-				uploadAction = ((CFolderScanItem*)pItem)->m_defaultFileExistsAction;
-			else if (((CFolderScanItem*)pItem)->m_defaultFileExistsAction != uploadAction)
-				upload_unknown = true;
-			has_upload = true;
-			break;
 		case QueueItemType::File:
 			{
 				CFileItem *pFileItem = (CFileItem*)pItem;
@@ -2465,11 +2033,6 @@ void CQueueView::OnSetDefaultFileExistsAction(wxCommandEvent &)
 
 		switch (pItem->GetType())
 		{
-		case QueueItemType::FolderScan:
-			if (!has_upload)
-				break;
-			((CFolderScanItem*)pItem)->m_defaultFileExistsAction = uploadAction;
-			break;
 		case QueueItemType::File:
 			{
 				CFileItem *pFileItem = (CFileItem*)pItem;
@@ -2534,8 +2097,7 @@ t_EngineData* CQueueView::GetIdleEngine(const CServer* pServer, bool allowTransi
 		const int newEngineCount = COptions::Get()->GetOptionVal(OPTION_NUMTRANSFERS);
 		if (newEngineCount > static_cast<int>(m_engineData.size()) - transient) {
 			pFirstIdle = new t_EngineData;
-			pFirstIdle->pEngine = new CFileZillaEngine(m_pMainFrame->GetEngineContext());
-			pFirstIdle->pEngine->Init(this);
+			pFirstIdle->pEngine = new CFileZillaEngine(m_pMainFrame->GetEngineContext(), *this);
 
 			m_engineData.push_back(pFirstIdle);
 		}
@@ -2620,7 +2182,7 @@ void CQueueView::TryRefreshListings()
 	}
 }
 
-void CQueueView::OnAskPassword(wxCommandEvent&)
+void CQueueView::OnAskPassword()
 {
 	while (!m_waitingForPassword.empty()) {
 		const CFileZillaEngine* const pEngine = m_waitingForPassword.front();
@@ -2756,16 +2318,6 @@ void CQueueView::OnTimer(wxTimerEvent& event)
 		return;
 	}
 
-	if (id == m_folderscan_item_refresh_timer.GetId()) {
-		if (m_queuedFolders[1].empty())
-			return;
-
-		CFolderScanItem* pItem = m_queuedFolders[1].front();
-		pItem->m_statusMessage = wxString::Format(_("%d files added to queue"), pItem->GetCount());
-		RefreshItem(pItem);
-		return;
-	}
-
 	for (auto & pData : m_engineData) {
 		if (pData->m_idleDisconnectTimer && !pData->m_idleDisconnectTimer->IsRunning()) {
 			delete pData->m_idleDisconnectTimer;
@@ -2901,19 +2453,30 @@ void CQueueView::ActionAfter(bool warned /*=false*/)
 	const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 	for (unsigned int i = 0; i < pStates->size(); ++i) {
 		CState *pState = (*pStates)[i];
-		CRecursiveOperation *pRecursiveOperationHandler;
-		if (!pState || !(pRecursiveOperationHandler = pState->GetRecursiveOperationHandler()))
+		if (!pState) {
 			continue;
+		}
 
-		if (pRecursiveOperationHandler->GetOperationMode() == CRecursiveOperation::recursive_download ||
-			pRecursiveOperationHandler->GetOperationMode() == CRecursiveOperation::recursive_download_flatten)
-		{
-			return;
+		auto localRecursiveOperation = pState->GetLocalRecursiveOperation();
+		if (localRecursiveOperation) {
+			if (localRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_transfer ||
+				localRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_transfer_flatten)
+			{
+				return;
+			}
+		}
+
+		auto remoteRecursiveOperation = pState->GetRemoteRecursiveOperation();
+		if (remoteRecursiveOperation) {
+			if (remoteRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_transfer ||
+				remoteRecursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_transfer_flatten)
+			{
+				return;
+			}
 		}
 	}
 
-	switch (m_actionAfterState)
-	{
+	switch (m_actionAfterState) {
 		case ActionAfterState::ShowNotification:
 		{
 			wxString const title = _("Transfers finished");
