@@ -11,34 +11,9 @@
 #include <wx/progdlg.h>
 
 #include "queue_storage.h"
+#include "local_recursive_operation.h"
+#include "notification.h"
 
-class CFolderProcessingEntry
-{
-public:
-	enum t_type
-	{
-		dir,
-		file
-	};
-	const t_type m_type;
-
-	CFolderProcessingEntry(t_type type) : m_type(type) {}
-	virtual ~CFolderProcessingEntry() {}
-};
-
-class t_newEntry : public CFolderProcessingEntry
-{
-public:
-	t_newEntry()
-		: CFolderProcessingEntry(CFolderProcessingEntry::file)
-	{}
-
-	wxString name;
-	int64_t size{};
-	fz::datetime time;
-	int attributes{};
-	bool dir{};
-};
 
 namespace ActionAfterState {
 enum type {
@@ -60,7 +35,7 @@ enum type {
 
 class CStatusLineCtrl;
 class CFileItem;
-struct t_EngineData
+struct t_EngineData final
 {
 	t_EngineData()
 		: pEngine()
@@ -106,7 +81,6 @@ struct t_EngineData
 
 class CMainFrame;
 class CStatusLineCtrl;
-class CFolderProcessingThread;
 class CAsyncRequestQueue;
 class CQueue;
 #if WITH_LIBDBUS
@@ -115,9 +89,8 @@ class CDesktopNotification;
 class wxNotificationMessage;
 #endif
 
-class CQueueView : public CQueueViewBase, public COptionChangeEventHandler
+class CQueueView : public CQueueViewBase, public COptionChangeEventHandler, private EngineNotificationHandler
 {
-	friend class CFolderProcessingThread;
 	friend class CQueueViewDropTarget;
 	friend class CQueueViewFailed;
 public:
@@ -131,9 +104,8 @@ public:
 		QueuePriority priority = QueuePriority::normal);
 
 	void QueueFile_Finish(const bool start); // Need to be called after QueueFile
-	bool QueueFiles(const bool queueOnly, const CLocalPath& localPath, const CRemoteDataObject& dataObject);
-	int QueueFiles(const std::list<CFolderProcessingEntry*> &entryList, bool queueOnly, bool download, CServerItem* pServerItem, const CFileExistsNotification::OverwriteAction defaultFileExistsAction);
-	bool QueueFolder(bool queueOnly, bool download, const CLocalPath& localPath, const CServerPath& remotePath, const CServer& server);
+	bool QueueFiles(const bool queueOnly, CLocalPath const& localPath, const CRemoteDataObject& dataObject);
+	bool QueueFiles(const bool queueOnly, CServer const& server, CLocalRecursiveOperation::listing const& listing);
 
 	bool empty() const;
 	int IsActive() const { return m_activeMode; }
@@ -141,7 +113,6 @@ public:
 	bool Quit();
 
 	// This sets the default file exists action for all files currently in queue.
-	// This includes queued folders which are yet to be processed
 	void SetDefaultFileExistsAction(CFileExistsNotification::OverwriteAction action, const TransferDirection direction);
 
 	void UpdateItemSize(CFileItem* pItem, int64_t size);
@@ -185,9 +156,6 @@ protected:
 	// whether it is allowed to start another transfer on that server item
 	bool CanStartTransfer(const CServerItem& server_item, struct t_EngineData *&pEngineData);
 
-	bool ProcessFolderItems(int type = -1);
-	void ProcessUploadFolderItems();
-
 	void ProcessReply(t_EngineData* pEngineData, COperationNotification const& notification);
 	void SendNextCommand(t_EngineData& engineData);
 
@@ -246,17 +214,6 @@ protected:
 	std::vector<t_EngineData*> m_engineData;
 	std::list<CStatusLineCtrl*> m_statusLineList;
 
-	/*
-	 * List of queued folders used to populate the queue.
-	 * Index 0 for downloads, index 1 for uploads.
-	 * For each type, only the first one can be active at any given time.
-	 */
-	std::list<CFolderScanItem*> m_queuedFolders[2];
-
-	void RemoveQueuedFolderItem(CFolderScanItem* pFolder);
-
-	CFolderProcessingThread *m_pFolderProcessingThread{};
-
 	/*
 	 * Don't update status line positions if m_waitStatusLineUpdate is true.
 	 * This assures we are updating the status line positions only once,
@@ -291,8 +248,6 @@ protected:
 
 	virtual void OnPostScroll();
 
-	wxTimer m_folderscan_item_refresh_timer;
-
 	int GetLineHeight();
 	int m_line_height;
 #ifdef __WXMSW__
@@ -315,10 +270,12 @@ protected:
 	// Unit is byte/s.
 	wxFileOffset GetCurrentSpeed(bool countDownload, bool countUpload);
 
+	virtual void OnEngineEvent(CFileZillaEngine* engine);
+	void DoOnEngineEvent(CFileZillaEngine* engine);
+
+	void OnAskPassword();
+
 	DECLARE_EVENT_TABLE()
-	void OnEngineEvent(wxFzEvent &event);
-	void OnFolderThreadComplete(wxCommandEvent& event);
-	void OnFolderThreadFiles(wxCommandEvent& event);
 	void OnChar(wxKeyEvent& event);
 
 	// Context menu handlers
@@ -328,7 +285,6 @@ protected:
 	void OnRemoveSelected(wxCommandEvent& event);
 	void OnSetDefaultFileExistsAction(wxCommandEvent& event);
 
-	void OnAskPassword(wxCommandEvent& event);
 	void OnTimer(wxTimerEvent& evnet);
 
 	void OnSetPriority(wxCommandEvent& event);
