@@ -352,6 +352,7 @@ BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_MENU(XRCID("ID_EDIT"), CRemoteListView::OnMenuEdit)
 	EVT_MENU(XRCID("ID_ENTER"), CRemoteListView::OnMenuEnter)
 	EVT_MENU(XRCID("ID_GETURL"), CRemoteListView::OnMenuGeturl)
+	EVT_MENU(XRCID("ID_GETURL_PASSWORD"), CRemoteListView::OnMenuGeturl)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CRemoteListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
@@ -378,13 +379,13 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState& state, CQueueView* p
 	AddColumn(_("Owner/Group"), wxLIST_FORMAT_LEFT, widths[5]);
 	LoadColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
 
-	InitSort(OPTION_REMOTEFILELIST_SORTORDER);
-
 	m_dirIcon = GetIconIndex(iconType::dir);
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
 	InitHeaderSortImageList();
 
+	InitSort(OPTION_REMOTEFILELIST_SORTORDER);
+
 	SetDirectoryListing(0);
 
 	SetDropTarget(new CRemoteListViewDropTarget(this));
@@ -446,14 +447,20 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 
 	m_indexMapping[0] = pDirectoryListing->GetCount();
 
-	std::list<unsigned int> added;
-
 	CFilterManager filter;
 	const wxString path = m_pDirectoryListing->path.GetPath();
 
 	CGenericFileData last = m_fileData.back();
 	m_fileData.pop_back();
 
+	bool const has_selections = GetSelectedItemCount() != 0;
+
+	std::vector<int> added_indexes;
+	if (has_selections) {
+		added_indexes.reserve(to_add);
+	}
+
+	CFileListCtrl<CGenericFileData>::CSortComparisonObject compare = GetSortComparisonObject();
 	for (unsigned int i = pDirectoryListing->GetCount() - to_add; i < pDirectoryListing->GetCount(); ++i) {
 		const CDirentry& entry = (*pDirectoryListing)[i];
 		CGenericFileData data;
@@ -480,43 +487,31 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 		std::vector<unsigned int>::iterator start = m_indexMapping.begin();
 		if (m_hasParent)
 			++start;
-		CFileListCtrl<CGenericFileData>::CSortComparisonObject compare = GetSortComparisonObject();
 		std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), i, compare);
-		compare.Destroy();
 
-		const int item = insertPos - m_indexMapping.begin();
+		const int added_index = insertPos - m_indexMapping.begin();
 		m_indexMapping.insert(insertPos, i);
 
-		for (auto iter = added.begin(); iter != added.end(); ++iter) {
-			unsigned int &pos = *iter;
-			if (pos >= (unsigned int)item)
-				++pos;
+		// Remember inserted index
+		if (has_selections) {
+			auto const added_indexes_insert_pos = std::lower_bound(added_indexes.begin(), added_indexes.end(), added_index);
+			for (auto index = added_indexes_insert_pos; index != added_indexes.end(); ++index) {
+				++(*index);
+			}
+			added_indexes.insert(added_indexes_insert_pos, added_index);
 		}
-		added.push_back(item);
 	}
+	compare.Destroy();
 
 	m_fileData.push_back(last);
 
 	std::list<bool> selected;
 	unsigned int start;
-	added.push_back(m_indexMapping.size());
-	start = added.front();
+	added_indexes.push_back(m_indexMapping.size());
+	start = added_indexes.front();
 
 	SetItemCount(m_indexMapping.size());
-
-	for (unsigned int i = start; i < m_indexMapping.size(); ++i) {
-		if (i == added.front()) {
-			selected.push_front(false);
-			added.pop_front();
-		}
-		bool is_selected = GetItemState(i, wxLIST_STATE_SELECTED) != 0;
-		selected.push_back(is_selected);
-
-		bool should_selected = selected.front();
-		selected.pop_front();
-		if (is_selected != should_selected)
-			SetSelection(i, should_selected);
-	}
+	UpdateSelections_ItemsAdded(added_indexes);
 
 	if (m_pFilelistStatusBar)
 		m_pFilelistStatusBar->SetHidden(m_pDirectoryListing->GetCount() + 1 - m_indexMapping.size());
@@ -660,10 +655,8 @@ bool CRemoteListView::UpdateDirectoryListing(std::shared_ptr<CDirectoryListing>
 	if (unsure & CDirectoryListing::unsure_invalid)
 		return false;
 
-	if (!(unsure & ~(CDirectoryListing::unsure_dir_changed | CDirectoryListing::unsure_file_changed)))
-	{
-		if (m_sortColumn && m_sortColumn != 2)
-		{
+	if (!(unsure & ~(CDirectoryListing::unsure_dir_changed | CDirectoryListing::unsure_file_changed))) {
+		if (m_sortColumn != 0 && m_sortColumn != 2) {
 			// If not sorted by file or type, changing file attributes can influence
 			// sort order.
 			return false;
@@ -1070,7 +1063,6 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 						canEnter = true;
 					}
 				}
-				canEnter = true;
 			}
 		}
 		else {
@@ -1159,6 +1151,8 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		}
 	}
 
+	pMenu->Delete(XRCID(wxGetKeyState(WXK_SHIFT) ? "ID_GETURL" : "ID_GETURL_PASSWORD"));
+
 	PopupMenu(pMenu);
 	delete pMenu;
 }
@@ -2037,6 +2031,10 @@ void CRemoteListView::SetInfoText()
 
 void CRemoteListView::OnBeginDrag(wxListEvent&)
 {
+	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+		return;
+	}
+
 	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
@@ -2268,8 +2266,7 @@ bool CRemoteListView::CanStartComparison()
 
 void CRemoteListView::StartComparison()
 {
-	if (m_sortDirection || m_sortColumn)
-	{
+	if (m_sortDirection || m_sortColumn != 0) {
 		wxASSERT(m_originalIndexMapping.empty());
 		SortList(0, 0);
 	}
@@ -2394,9 +2391,9 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 		{
 			const CDirentry& entry = (*m_pDirectoryListing)[index];
 			if (m_pDirectoryListing->path.GetType() == VMS)
-				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir());
+				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir()).ToStdWstring();
 			else
-				data.fileType = GetType(entry.name, entry.is_dir());
+				data.fileType = GetType(entry.name, entry.is_dir()).ToStdWstring();
 		}
 
 		return data.fileType;
@@ -2419,8 +2416,7 @@ CFileListCtrl<CGenericFileData>::CSortComparisonObject CRemoteListView::GetSortC
 	CFileListCtrlSort<CDirectoryListing>::NameSortMode nameSortMode = GetNameSortMode();
 
 	CDirectoryListing const& directoryListing = *m_pDirectoryListing;
-	if (!m_sortDirection)
-	{
+	if (!m_sortDirection) {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -2434,8 +2430,7 @@ CFileListCtrl<CGenericFileData>::CSortComparisonObject CRemoteListView::GetSortC
 		else
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortName<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
 	}
-	else
-	{
+	else {
 		if (m_sortColumn == 1)
 			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
@@ -2482,22 +2477,22 @@ void CRemoteListView::LinkIsNotDir(const CServerPath& path, const wxString& link
 	m_pLinkResolveState.reset();
 }
 
-void CRemoteListView::OnMenuGeturl(wxCommandEvent&)
+void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 {
-	if (!m_pDirectoryListing)
+	if (!m_pDirectoryListing) {
 		return;
+	}
 
 	const CServer* pServer = m_state.GetServer();
-	if (!pServer)
+	if (!pServer) {
 		return;
+	}
 
 	long item = -1;
 
 	std::list<CDirentry> selected_item_list;
-	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
-		if (!item)
-		{
+	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
+		if (!item) {
 			wxBell();
 			return;
 		}
@@ -2510,49 +2505,47 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent&)
 
 		selected_item_list.push_back(entry);
 	}
-	if (selected_item_list.empty())
-	{
+	if (selected_item_list.empty()) {
 		wxBell();
 		return;
 	}
 
-	if (!wxTheClipboard->Open())
-	{
+	if (!wxTheClipboard->Open()) {
 		wxMessageBoxEx(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
 		return;
 	}
 
 	const CServerPath& path = m_pDirectoryListing->path;
-	const wxString server = pServer->FormatServer(true);
-	if (selected_item_list.size() == 1)
-	{
-		wxString url = server;
-		url += path.FormatFilename(selected_item_list.front().name, false);
 
-		// Poor mans URLencode
-		url.Replace(_T(" "), _T("%20"));
+	wxString const server = pServer->Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
-		wxTheClipboard->SetData(new wxURLDataObject(url));
+	auto getUrl = [](wxString const& serverPart, CServerPath const& path, wxString const& name) {
+		wxString url = serverPart;
+
+		auto const pathPart = url_encode(path.FormatFilename(name, false).ToStdWstring(), true);
+		if (!pathPart.empty() && pathPart[0] != '/') {
+			url += '/';
+		}
+		url += pathPart;
+
+		return url;
+	};
+
+	wxString urls;
+	if (selected_item_list.size() == 1) {
+		urls = getUrl(server, path, selected_item_list.front().name);
 	}
-	else
-	{
-		wxString urls;
-		for (std::list<CDirentry>::const_iterator iter = selected_item_list.begin(); iter != selected_item_list.end(); ++iter)
-		{
-			urls += server;
-			urls += path.FormatFilename(iter->name, false);
+	else {
+		for (auto const& entry : selected_item_list) {
+			urls = getUrl(server, path, entry.name);
 #ifdef __WXMSW__
 			urls += _T("\r\n");
 #else
 			urls += _T("\n");
 #endif
 		}
-
-		// Poor mans URLencode
-		urls.Replace(_T(" "), _T("%20"));
-
-		wxTheClipboard->SetData(new wxTextDataObject(urls));
 	}
+	wxTheClipboard->SetData(new wxURLDataObject(urls));
 
 	wxTheClipboard->Flush();
 	wxTheClipboard->Close();
