@@ -772,6 +772,10 @@ CServerPath CRemoteTreeView::GetPathFromItem(const wxTreeItemId& item) const
 
 void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 {
+	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
+		return;
+	}
+
 	// Drag could result in recursive operation, don't allow at this point
 	if (!m_state.IsRemoteIdle()) {
 		wxBell();
@@ -884,6 +888,8 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
 
+	pMenu->Delete(XRCID(wxGetKeyState(WXK_SHIFT) ? "ID_GETURL" : "ID_GETURL_PASSWORD"));
+
 	PopupMenu(pMenu);
 	delete pMenu;
 }
@@ -1357,36 +1363,35 @@ void CRemoteTreeView::ApplyFilters(bool resort)
 	}
 }
 
-void CRemoteTreeView::OnMenuGeturl(wxCommandEvent&)
+void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 {
 	if (!m_contextMenuItem)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.empty())
-	{
+	if (path.empty()) {
 		wxBell();
 		return;
 	}
 
 	const CServer *pServer = m_state.GetServer();
-	if (!pServer)
-	{
+	if (!pServer) {
 		wxBell();
 		return;
 	}
 
-	if (!wxTheClipboard->Open())
-	{
+	if (!wxTheClipboard->Open()) {
 		wxMessageBoxEx(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	wxString url = pServer->FormatServer(true);
-	url += path.GetPath();
+	wxString url = pServer->Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
-	// Poor mans URLencode
-	url.Replace(_T(" "), _T("%20"));
+	wxString const pathPart = url_encode(path.GetPath().ToStdWstring(), true);
+	if (!pathPart.empty() && pathPart[0] != '/') {
+		url += '/';
+	}
+	url += pathPart;
 
 	wxTheClipboard->SetData(new wxURLDataObject(url));
 
