@@ -7,15 +7,17 @@
 
 #include <wx/timer.h>
 
+#include "option_change_event_handler.h"
+
 class CFastTextCtrl;
-class CStatusView final : public wxNavigationEnabled<wxWindow>
+class CStatusView final : public wxNavigationEnabled<wxWindow>, private COptionChangeEventHandler
 {
 public:
 	CStatusView(wxWindow* parent, wxWindowID id);
 	virtual ~CStatusView();
 
 	void AddToLog(CLogmsgNotification const& pNotification);
-	void AddToLog(MessageType messagetype, const wxString& message, CDateTime const& time);
+	void AddToLog(MessageType messagetype, const wxString& message, fz::datetime const& time);
 
 	void InitDefAttr();
 
@@ -23,12 +25,14 @@ public:
 
 	virtual bool Show(bool show = true);
 
-protected:
+private:
 
 	int m_nLineCount{};
 	wxString m_Content;
 	CFastTextCtrl *m_pTextCtrl{};
 
+	void OnOptionsChanged(changed_options_t const& options);
+
 	DECLARE_EVENT_TABLE()
 	void OnSize(wxSizeEvent &);
 	void OnContextMenu(wxContextMenuEvent&);
@@ -59,12 +63,12 @@ protected:
 	{
 		MessageType messagetype;
 		wxString message;
-		CDateTime time;
+		fz::datetime time;
 	};
 	std::list<t_line> m_hiddenLines;
 
 	bool m_showTimestamps{};
-	CDateTime m_lastTime;
+	fz::datetime m_lastTime;
 	wxString m_lastTimeString;
 };
 
