@@ -2,20 +2,22 @@
 #include "commandqueue.h"
 #include "Mainfrm.h"
 #include "state.h"
-#include "recursive_operation.h"
+#include "remote_recursive_operation.h"
 #include "loginmanager.h"
 #include "queue.h"
 #include "RemoteListView.h"
 
+#include <algorithm>
+
 DEFINE_EVENT_TYPE(fzEVT_GRANTEXCLUSIVEENGINEACCESS)
 
 int CCommandQueue::m_requestIdCounter = 0;
 
-CCommandQueue::CCommandQueue(CFileZillaEngine *pEngine, CMainFrame* pMainFrame, CState* pState)
+CCommandQueue::CCommandQueue(CFileZillaEngine *pEngine, CMainFrame* pMainFrame, CState& state)
+	: m_state(state)
 {
 	m_pEngine = pEngine;
 	m_pMainFrame = pMainFrame;
-	m_pState = pState;
 	m_exclusiveEngineRequest = false;
 	m_exclusiveEngineLock = false;
 	m_requestId = 0;
@@ -25,20 +27,30 @@ CCommandQueue::~CCommandQueue()
 {
 }
 
-bool CCommandQueue::Idle() const
+bool CCommandQueue::Idle(command_origin origin) const
 {
-	return m_CommandList.empty() && !m_exclusiveEngineLock;
-}
-
-void CCommandQueue::ProcessCommand(CCommand *pCommand)
-{
-	if (m_quit) {
-		delete pCommand;
+	if (m_exclusiveEngineLock) {
+		return false;
 	}
 
-	m_CommandList.emplace_back(pCommand);
+	if (origin == any) {
+		return m_CommandList.empty();
+	}
+	
+	return std::find_if(m_CommandList.begin(), m_CommandList.end(), [origin](CommandInfo const& c) { return c.origin == origin; }) == m_CommandList.end();
+}
+
+void CCommandQueue::ProcessCommand(CCommand *pCommand, CCommandQueue::command_origin origin)
+{
+	wxASSERT(origin != any);
+	if (m_quit) {
+		delete pCommand;
+		return;
+	}
+
+	m_CommandList.emplace_back(origin, std::unique_ptr<CCommand>(pCommand));
 	if (m_CommandList.size() == 1) {
-		m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+		m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
 		ProcessNextCommand();
 	}
 }
@@ -63,14 +75,14 @@ void CCommandQueue::ProcessNextCommand()
 		// - Interface cannot obtain listing since not connected
 		// - Yet getting operation successful
 		// To keep things flowing, we need to advance the recursive operation.
-		m_pState->GetRecursiveOperationHandler()->NextOperation();
+		m_state.GetRemoteRecursiveOperation()->NextOperation();
 	}
 
 	while (!m_CommandList.empty()) {
-		std::unique_ptr<CCommand> const& pCommand = m_CommandList.front();
+		auto const& commandInfo = m_CommandList.front();
 
-		int res = m_pEngine->Execute(*pCommand);
-		ProcessReply(res, pCommand->GetId());
+		int res = m_pEngine->Execute(*commandInfo.command);
+		ProcessReply(res, commandInfo.command->GetId());
 		if (res == FZ_REPLY_WOULDBLOCK) {
 			break;
 		}
@@ -82,10 +94,10 @@ void CCommandQueue::ProcessNextCommand()
 		if (m_exclusiveEngineRequest)
 			GrantExclusiveEngineRequest();
 		else
-			m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+			m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
 
-		if (!m_pState->SuccessfulConnect())
-			m_pState->SetServer(0);
+		if (!m_state.SuccessfulConnect())
+			m_state.SetServer(0);
 	}
 }
 
@@ -101,7 +113,7 @@ bool CCommandQueue::Cancel()
 
 	if (!m_pEngine)	{
 		m_CommandList.clear();
-		m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+		m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
 		return true;
 	}
 
@@ -110,7 +122,7 @@ bool CCommandQueue::Cancel()
 		return false;
 	else {
 		m_CommandList.clear();
-		m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+		m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
 		return true;
 	}
 }
@@ -136,7 +148,7 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 			return;
 		}
 		if (nReplyCode & FZ_REPLY_PASSWORDFAILED)
-			CLoginManager::Get().CachedPasswordFailed(*m_pState->GetServer());
+			CLoginManager::Get().CachedPasswordFailed(*m_state.GetServer());
 	}
 
 	if (m_CommandList.empty()) {
@@ -144,13 +156,27 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 	}
 
 	if (commandId != Command::connect &&
-		commandId != Command::disconnect)
+		commandId != Command::disconnect &&
+		(nReplyCode & FZ_REPLY_CANCELED) != FZ_REPLY_CANCELED)
 	{
+		bool reconnect = false;
 		if (nReplyCode == FZ_REPLY_NOTCONNECTED) {
+			reconnect = true;
+		}
+		else if (nReplyCode & FZ_REPLY_DISCONNECTED) {
+			auto & info = m_CommandList.front();
+			if (!info.didReconnect) {
+				info.didReconnect = true;
+				reconnect = true;
+			}
+		}
+
+		if (reconnect) {
 			// Try automatic reconnect
-			const CServer* pServer = m_pState->GetServer();
+			const CServer* pServer = m_state.GetServer();
 			if (pServer) {
-				m_CommandList.emplace_front(make_unique<CConnectCommand>(*pServer));
+				m_CommandList.emplace_front(normal, std::make_unique<CConnectCommand>(*pServer));
+				ProcessNextCommand();
 				return;
 			}
 		}
@@ -158,39 +184,46 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 
 	++m_inside_commandqueue;
 
-	auto const& pCommand = m_CommandList.front();
+	auto const& commandInfo = m_CommandList.front();
 
-	if (pCommand->GetId() == Command::list && nReplyCode != FZ_REPLY_OK) {
-		if (nReplyCode & FZ_REPLY_LINKNOTDIR)
-		{
+	if (commandInfo.command->GetId() == Command::list && nReplyCode != FZ_REPLY_OK) {
+		if (nReplyCode & FZ_REPLY_LINKNOTDIR) {
 			// Symbolic link does not point to a directory. Either points to file
 			// or is completely invalid
-			CListCommand* pListCommand = static_cast<CListCommand*>(pCommand.get());
+			CListCommand* pListCommand = static_cast<CListCommand*>(commandInfo.command.get());
 			wxASSERT(pListCommand->GetFlags() & LIST_FLAG_LINK);
 
-			m_pState->LinkIsNotDir(pListCommand->GetPath(), pListCommand->GetSubDir());
+			m_state.LinkIsNotDir(pListCommand->GetPath(), pListCommand->GetSubDir());
+		}
+		else {
+			if (commandInfo.origin == recursiveOperation) {
+				// Let the recursive operation handler know if a LIST command failed,
+				// so that it may issue the next command in recursive operations.
+				m_state.GetRemoteRecursiveOperation()->ListingFailed(nReplyCode);
+			}
+			else {
+				m_state.ListingFailed(nReplyCode);
+			}
 		}
-		else
-			m_pState->ListingFailed(nReplyCode);
 		m_CommandList.pop_front();
 	}
-	else if (nReplyCode == FZ_REPLY_ALREADYCONNECTED && pCommand->GetId() == Command::connect) {
-		m_CommandList.emplace_front(make_unique<CDisconnectCommand>());
+	else if (nReplyCode == FZ_REPLY_ALREADYCONNECTED && commandInfo.command->GetId() == Command::connect) {
+		m_CommandList.emplace_front(normal, std::make_unique<CDisconnectCommand>());
 	}
-	else if (pCommand->GetId() == Command::connect && nReplyCode != FZ_REPLY_OK) {
+	else if (commandInfo.command->GetId() == Command::connect && nReplyCode != FZ_REPLY_OK) {
 		// Remove pending events
 		auto it = ++m_CommandList.begin();
-		while (it != m_CommandList.end() && (*it)->GetId() != Command::connect) {
+		while (it != m_CommandList.end() && it->command->GetId() != Command::connect) {
 			++it;
 		}
 		m_CommandList.erase(m_CommandList.begin(), it);
 
 		// If this was an automatic reconnect during a recursive
 		// operation, stop the recursive operation
-		m_pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
+		m_state.GetRemoteRecursiveOperation()->StopRecursiveOperation();
 	}
-	else if (pCommand->GetId() == Command::connect && nReplyCode == FZ_REPLY_OK) {
-		m_pState->SetSuccessfulConnect();
+	else if (commandInfo.command->GetId() == Command::connect && nReplyCode == FZ_REPLY_OK) {
+		m_state.SetSuccessfulConnect();
 		m_CommandList.pop_front();
 	}
 	else
@@ -215,7 +248,7 @@ void CCommandQueue::RequestExclusiveEngine(bool requestExclusive)
 		}
 		if (m_CommandList.empty())
 		{
-			m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+			m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
 			GrantExclusiveEngineRequest();
 			return;
 		}
@@ -223,7 +256,7 @@ void CCommandQueue::RequestExclusiveEngine(bool requestExclusive)
 	if (!requestExclusive)
 		m_exclusiveEngineLock = false;
 	m_exclusiveEngineRequest = requestExclusive;
-	m_pState->NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+	m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
 }
 
 void CCommandQueue::GrantExclusiveEngineRequest()
@@ -260,4 +293,32 @@ bool CCommandQueue::Quit()
 {
 	m_quit = true;
 	return Cancel();
-}
\ No newline at end of file
+}
+
+void CCommandQueue::ProcessDirectoryListing(CDirectoryListingNotification const& listingNotification)
+{
+	auto const firstListing = std::find_if(m_CommandList.begin(), m_CommandList.end(), [](CommandInfo const& v) { return v.command->GetId() == Command::list; });
+	bool const listingIsRecursive = firstListing != m_CommandList.end() && firstListing->origin == recursiveOperation;
+
+	std::shared_ptr<CDirectoryListing> pListing;
+	if (!listingNotification.GetPath().empty()) {
+		pListing = std::make_shared<CDirectoryListing>();
+		if (listingNotification.Failed() ||
+			m_state.m_pEngine->CacheLookup(listingNotification.GetPath(), *pListing) != FZ_REPLY_OK)
+		{
+			pListing = std::make_shared<CDirectoryListing>();
+			pListing->path = listingNotification.GetPath();
+			pListing->m_flags |= CDirectoryListing::listing_failed;
+			pListing->m_firstListTime = fz::monotonic_clock::now();
+		}
+	}
+
+	if (listingIsRecursive) {
+		if (!listingNotification.Modified() && m_state.GetRemoteRecursiveOperation()->IsActive()) {
+			m_state.NotifyHandlers(STATECHANGE_REMOTE_DIR_OTHER, wxString(), &pListing);
+		}
+	}
+	else {
+		m_state.SetRemoteDir(pListing, listingNotification.Modified());
+	}
+}
