@@ -3,12 +3,14 @@
 #include "context_control.h"
 #include "filelist_statusbar.h"
 #include "filezillaapp.h"
+#include "local_recursive_operation.h"
 #include "LocalListView.h"
 #include "LocalTreeView.h"
 #include "Mainfrm.h"
 #include "Options.h"
 #include "queue.h"
-#include "recursive_operation.h"
+#include "remote_recursive_operation.h"
+#include "recursive_operation_status.h"
 #include "RemoteListView.h"
 #include "RemoteTreeView.h"
 #include "sitemanager.h"
@@ -29,16 +31,12 @@ EVT_MENU(XRCID("ID_TABCONTEXT_CLOSEOTHERS"), CContextControl::OnTabContextCloseO
 EVT_MENU(XRCID("ID_TABCONTEXT_NEW"), CContextControl::OnTabContextNew)
 END_EVENT_TABLE()
 
-CContextControl::CContextControl(CMainFrame* pMainFrame)
-	: CStateEventHandler(0),
-	m_tabs(0), m_right_clicked_tab(-1), m_pMainFrame(pMainFrame)
+CContextControl::CContextControl(CMainFrame& mainFrame)
+	: m_mainFrame(mainFrame)
 {
-	m_current_context_controls = -1;
-	m_tabs = 0;
-
 	wxASSERT(!CContextManager::Get()->HandlerCount(STATECHANGE_CHANGEDCONTEXT));
-	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false, false);
-	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, false, false);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, false);
 }
 
 CContextControl::~CContextControl()
@@ -53,60 +51,68 @@ void CContextControl::Create(wxWindow *parent)
 void CContextControl::CreateTab()
 {
 	wxGetApp().AddStartupProfileRecord(_T("CContextControl::CreateTab"));
-#ifdef __WXMSW__
-	// Some reparenting is being done when creating tabs. Reparenting of frozen windows isn't working
-	// on OS X.
-	wxWindowUpdateLocker lock(this);
-#endif
 
-	CState* pState = 0;
+	{
+	#ifdef __WXMSW__
+		// Some reparenting is being done when creating tabs. Reparenting of frozen windows isn't working
+		// on OS X.
+		wxWindowUpdateLocker lock(this);
+	#endif
 
-	// See if we can reuse an existing context
-	for (size_t i = 0; i < m_context_controls.size(); i++) {
-		if (m_context_controls[i].tab_index != -1)
-			continue;
+		CState* pState = 0;
 
-		if (m_context_controls[i].pState->IsRemoteConnected() ||
-			!m_context_controls[i].pState->IsRemoteIdle())
-			continue;
+		// See if we can reuse an existing context
+		for (size_t i = 0; i < m_context_controls.size(); i++) {
+			if (m_context_controls[i].tab_index != -1)
+				continue;
 
-		pState = m_context_controls[i].pState;
-		m_context_controls.erase(m_context_controls.begin() + i);
-		if (m_current_context_controls > (int)i)
-			m_current_context_controls--;
-		break;
-	}
-	if (!pState) {
-		pState = CContextManager::Get()->CreateState(m_pMainFrame);
-		if (!pState->CreateEngine()) {
-			wxMessageBoxEx(_("Failed to initialize FTP engine"));
+			if (m_context_controls[i].pState->IsRemoteConnected() ||
+				!m_context_controls[i].pState->IsRemoteIdle())
+				continue;
+
+			pState = m_context_controls[i].pState;
+			m_context_controls.erase(m_context_controls.begin() + i);
+			if (m_current_context_controls > (int)i)
+				m_current_context_controls--;
+			break;
 		}
+		if (!pState) {
+			pState = CContextManager::Get()->CreateState(m_mainFrame);
+			if (!pState->CreateEngine()) {
+				wxMessageBoxEx(_("Failed to initialize FTP engine"));
+			}
+		}
+
+		// Restore last server and path
+		CServer last_server;
+		CServerPath last_path;
+		if (COptions::Get()->GetLastServer(last_server) && last_path.SetSafePath(COptions::Get()->GetOption(OPTION_LASTSERVERPATH)))
+			pState->SetLastServer(last_server, last_path);
+
+		CreateContextControls(*pState);
+
+		pState->GetLocalRecursiveOperation()->SetQueue(m_mainFrame.GetQueue());
+		pState->GetRemoteRecursiveOperation()->SetQueue(m_mainFrame.GetQueue());
+
+		wxString const localDir = COptions::Get()->GetOption(OPTION_LASTLOCALDIR);
+		if (!pState->SetLocalDir(localDir)) {
+			wxString const homeDir = wxGetHomeDir();
+			if (!pState->SetLocalDir(homeDir)) {
+				pState->SetLocalDir(_T("/"));
+			}
+		}
+
+		CContextManager::Get()->SetCurrentContext(pState);
 	}
 
-	// Restore last server and path
-	CServer last_server;
-	CServerPath last_path;
-	if (COptions::Get()->GetLastServer(last_server) && last_path.SetSafePath(COptions::Get()->GetOption(OPTION_LASTSERVERPATH)))
-		pState->SetLastServer(last_server, last_path);
-
-	CreateContextControls(pState);
-
-	pState->GetRecursiveOperationHandler()->SetQueue(m_pMainFrame->GetQueue());
-
-	wxString localDir = COptions::Get()->GetOption(OPTION_LASTLOCALDIR);
-	if (!pState->SetLocalDir(localDir))
-		pState->SetLocalDir(_T("/"));
-
-	CContextManager::Get()->SetCurrentContext(pState);
-
-	if (!m_pMainFrame->RestoreSplitterPositions())
-		m_pMainFrame->SetDefaultSplitterPositions();
+	if (!m_mainFrame.RestoreSplitterPositions())
+		m_mainFrame.SetDefaultSplitterPositions();
 
 	if (m_tabs)
 		m_tabs->SetSelection(m_tabs->GetPageCount() - 1);
 }
 
-void CContextControl::CreateContextControls(CState* pState)
+void CContextControl::CreateContextControls(CState& state)
 {
 	wxGetApp().AddStartupProfileRecord(_T("CContextControl::CreateContextControls"));
 	wxWindow* parent = this;
@@ -145,7 +151,7 @@ void CContextControl::CreateContextControls(CState* pState)
 #endif
 		}
 
-		m_pMainFrame->RememberSplitterPositions();
+		m_mainFrame.RememberSplitterPositions();
 		m_context_controls[m_current_context_controls].pLocalListView->SaveColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
 		m_context_controls[m_current_context_controls].pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
 
@@ -154,7 +160,7 @@ void CContextControl::CreateContextControls(CState* pState)
 
 	struct CContextControl::_context_controls context_controls;
 
-	context_controls.pState = pState;
+	context_controls.pState = &state;
 	context_controls.pViewSplitter = new CSplitterWindowEx(parent, -1, initial_position, wxDefaultSize, wxSP_NOBORDER  | wxSP_LIVE_UPDATE);
 	context_controls.pViewSplitter->SetMinimumPaneSize(50, 100);
 	context_controls.pViewSplitter->SetSashGravity(0.5);
@@ -167,15 +173,15 @@ void CContextControl::CreateContextControls(CState* pState)
 
 	context_controls.pLocalTreeViewPanel = new CView(context_controls.pLocalSplitter);
 	context_controls.pLocalListViewPanel = new CView(context_controls.pLocalSplitter);
-	context_controls.pLocalTreeView = new CLocalTreeView(context_controls.pLocalTreeViewPanel, -1, pState, m_pMainFrame->GetQueue());
-	context_controls.pLocalListView = new CLocalListView(context_controls.pLocalListViewPanel, pState, m_pMainFrame->GetQueue());
+	context_controls.pLocalTreeView = new CLocalTreeView(context_controls.pLocalTreeViewPanel, -1, state, m_mainFrame.GetQueue());
+	context_controls.pLocalListView = new CLocalListView(context_controls.pLocalListViewPanel, state, m_mainFrame.GetQueue());
 	context_controls.pLocalTreeViewPanel->SetWindow(context_controls.pLocalTreeView);
 	context_controls.pLocalListViewPanel->SetWindow(context_controls.pLocalListView);
 
 	context_controls.pRemoteTreeViewPanel = new CView(context_controls.pRemoteSplitter);
 	context_controls.pRemoteListViewPanel = new CView(context_controls.pRemoteSplitter);
-	context_controls.pRemoteTreeView = new CRemoteTreeView(context_controls.pRemoteTreeViewPanel, -1, pState, m_pMainFrame->GetQueue());
-	context_controls.pRemoteListView = new CRemoteListView(context_controls.pRemoteListViewPanel, pState, m_pMainFrame->GetQueue());
+	context_controls.pRemoteTreeView = new CRemoteTreeView(context_controls.pRemoteTreeViewPanel, -1, state, m_mainFrame.GetQueue());
+	context_controls.pRemoteListView = new CRemoteListView(context_controls.pRemoteListViewPanel, state, m_mainFrame.GetQueue());
 	context_controls.pRemoteTreeViewPanel->SetWindow(context_controls.pRemoteTreeView);
 	context_controls.pRemoteListViewPanel->SetWindow(context_controls.pRemoteListView);
 
@@ -194,6 +200,11 @@ void CContextControl::CreateContextControls(CState* pState)
 	context_controls.pRemoteListViewPanel->SetStatusBar(pRemoteFilelistStatusBar);
 	context_controls.pRemoteListView->SetFilelistStatusBar(pRemoteFilelistStatusBar);
 
+	auto localRecursiveStatus = new CRecursiveOperationStatus(context_controls.pLocalListViewPanel, state, true);
+	context_controls.pLocalListViewPanel->SetFooter(localRecursiveStatus);
+
+	auto remoteRecursiveStatus = new CRecursiveOperationStatus(context_controls.pRemoteListViewPanel, state, false);
+	context_controls.pRemoteListViewPanel->SetFooter(remoteRecursiveStatus);
 
 	const int layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
 	const int swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
@@ -212,7 +223,7 @@ void CContextControl::CreateContextControls(CState* pState)
 	}
 
 	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_LOCAL)) {
-		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalTreeViewPanel, pState);
+		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalTreeViewPanel, state);
 		context_controls.pLocalTreeViewPanel->SetHeader(context_controls.pLocalViewHeader);
 		if (layout == 3 && swap)
 			context_controls.pLocalSplitter->SplitVertically(context_controls.pLocalListViewPanel, context_controls.pLocalTreeViewPanel);
@@ -223,13 +234,13 @@ void CContextControl::CreateContextControls(CState* pState)
 	}
 	else {
 		context_controls.pLocalTreeViewPanel->Hide();
-		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalListViewPanel, pState);
+		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalListViewPanel, state);
 		context_controls.pLocalListViewPanel->SetHeader(context_controls.pLocalViewHeader);
 		context_controls.pLocalSplitter->Initialize(context_controls.pLocalListViewPanel);
 	}
 
 	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_REMOTE)) {
-		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteTreeViewPanel, pState);
+		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteTreeViewPanel, state);
 		context_controls.pRemoteTreeViewPanel->SetHeader(context_controls.pRemoteViewHeader);
 		if (layout == 3 && !swap)
 			context_controls.pRemoteSplitter->SplitVertically(context_controls.pRemoteListViewPanel, context_controls.pRemoteTreeViewPanel);
@@ -240,7 +251,7 @@ void CContextControl::CreateContextControls(CState* pState)
 	}
 	else {
 		context_controls.pRemoteTreeViewPanel->Hide();
-		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteListViewPanel, pState);
+		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteListViewPanel, state);
 		context_controls.pRemoteListViewPanel->SetHeader(context_controls.pRemoteViewHeader);
 		context_controls.pRemoteSplitter->Initialize(context_controls.pRemoteListViewPanel);
 	}
@@ -252,21 +263,21 @@ void CContextControl::CreateContextControls(CState* pState)
 			context_controls.pLocalSplitter->SetSashGravity(1.0);
 	}
 
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pLocalListView);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pRemoteListView);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pLocalTreeView);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pRemoteTreeView);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pLocalViewHeader);
-	m_pMainFrame->ConnectNavigationHandler(context_controls.pRemoteViewHeader);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalListView);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pRemoteListView);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalTreeView);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pRemoteTreeView);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pLocalViewHeader);
+	m_mainFrame.ConnectNavigationHandler(context_controls.pRemoteViewHeader);
 
-	pState->GetComparisonManager()->SetListings(context_controls.pLocalListView, context_controls.pRemoteListView);
+	state.GetComparisonManager()->SetListings(context_controls.pLocalListView, context_controls.pRemoteListView);
 
 	if (m_tabs) {
 		context_controls.tab_index = m_tabs->GetPageCount();
-		m_tabs->AddPage(context_controls.pViewSplitter, pState->GetTitle());
+		m_tabs->AddPage(context_controls.pViewSplitter, state.GetTitle());
 
 		// Copy reconnect and bookmark information
-		pState->SetLastServer(
+		state.SetLastServer(
 			m_context_controls[m_current_context_controls].pState->GetLastServer(),
 			m_context_controls[m_current_context_controls].pState->GetLastServerPath());
 
@@ -353,12 +364,12 @@ bool CContextControl::CloseTab(int tab)
 #endif
 
 	pState->m_pCommandQueue->Cancel();
-	pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
+	pState->GetLocalRecursiveOperation()->StopRecursiveOperation();
+	pState->GetRemoteRecursiveOperation()->StopRecursiveOperation();
 
 	pState->GetComparisonManager()->SetListings(0, 0);
 
-	if (m_tabs->GetPageCount() == 2)
-	{
+	if (m_tabs->GetPageCount() == 2) {
 		// Get rid again of tab bar
 		m_tabs->Disconnect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED, wxAuiNotebookEventHandler(CContextControl::OnTabChanged), 0, this);
 
@@ -366,10 +377,10 @@ bool CContextControl::CloseTab(int tab)
 		m_tabs->RemovePage(keep);
 
 		size_t j;
-		for (j = 0; j < m_context_controls.size(); j++)
-		{
-			if (m_context_controls[j].tab_index != keep)
+		for (j = 0; j < m_context_controls.size(); ++j) {
+			if (m_context_controls[j].tab_index != keep) {
 				continue;
+			}
 
 			break;
 		}
@@ -389,26 +400,25 @@ bool CContextControl::CloseTab(int tab)
 
 		tabs->Destroy();
 	}
-	else
-	{
-		if (pState == CContextManager::Get()->GetCurrentContext())
-		{
+	else {
+		if (pState == CContextManager::Get()->GetCurrentContext()) {
 			int newsel = tab + 1;
-			if (newsel >= (int)m_tabs->GetPageCount())
+			if (newsel >= (int)m_tabs->GetPageCount()) {
 				newsel = m_tabs->GetPageCount() - 2;
+			}
 
-			for (size_t j = 0; j < m_context_controls.size(); j++)
-			{
-				if (m_context_controls[j].tab_index != newsel)
+			for (size_t j = 0; j < m_context_controls.size(); ++j) {
+				if (m_context_controls[j].tab_index != newsel) {
 					continue;
+				}
 				m_tabs->SetSelection(newsel);
 				CContextManager::Get()->SetCurrentContext(m_context_controls[j].pState);
 			}
 		}
-		for (size_t j = 0; j < m_context_controls.size(); j++)
-		{
-			if (m_context_controls[j].tab_index > tab)
-				m_context_controls[j].tab_index--;
+		for (size_t j = 0; j < m_context_controls.size(); ++j) {
+			if (m_context_controls[j].tab_index > tab) {
+				--m_context_controls[j].tab_index;
+			}
 		}
 		m_context_controls[i].tab_index = -1;
 		m_context_controls[i].site_bookmarks.reset();
@@ -428,14 +438,12 @@ void CContextControl::OnTabBgDoubleclick(wxAuiNotebookEvent&)
 void CContextControl::OnTabRightclick(wxAuiNotebookEvent& event)
 {
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_TABCONTEXT"));
-	if (!pMenu)
-	{
+	if (!pMenu) {
 		wxBell();
 		return;
 	}
 
-	if (!m_tabs || m_tabs->GetPageCount() < 2)
-	{
+	if (!m_tabs || m_tabs->GetPageCount() < 2) {
 		pMenu->Enable(XRCID("ID_TABCONTEXT_CLOSE"), false);
 		pMenu->Enable(XRCID("ID_TABCONTEXT_CLOSEOTHERS"), false);
 	}
@@ -447,7 +455,7 @@ void CContextControl::OnTabRightclick(wxAuiNotebookEvent& event)
 	delete pMenu;
 }
 
-void CContextControl::OnTabContextClose(wxCommandEvent& event)
+void CContextControl::OnTabContextClose(wxCommandEvent&)
 {
 	if (m_right_clicked_tab == -1)
 		return;
@@ -457,7 +465,7 @@ void CContextControl::OnTabContextClose(wxCommandEvent& event)
 	QueueEvent(new wxCommandEvent(fzEVT_TAB_CLOSING_DEFERRED, m_right_clicked_tab));
 }
 
-void CContextControl::OnTabContextCloseOthers(wxCommandEvent& event)
+void CContextControl::OnTabContextCloseOthers(wxCommandEvent&)
 {
 	QueueEvent(new wxCommandEvent (fzEVT_TAB_CLOSING_DEFERRED, -m_right_clicked_tab - 1));
 }
@@ -555,27 +563,23 @@ void CContextControl::AdvanceTab(bool forward)
 	m_tabs->AdvanceTab(forward);
 }
 
-void CContextControl::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+void CContextControl::OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*)
 {
-	if (notification == STATECHANGE_CHANGEDCONTEXT)
-	{
-		if (!pState)
-		{
+	if (notification == STATECHANGE_CHANGEDCONTEXT) {
+		if (!pState) {
 			m_current_context_controls = 0;
 			return;
 		}
 
 		// Get current controls for new current context
-		for (m_current_context_controls = 0; m_current_context_controls < (int)m_context_controls.size(); m_current_context_controls++)
-		{
+		for (m_current_context_controls = 0; m_current_context_controls < (int)m_context_controls.size(); ++m_current_context_controls) {
 			if (m_context_controls[m_current_context_controls].pState == pState)
 				break;
 		}
 		if (m_current_context_controls == (int)m_context_controls.size())
 			m_current_context_controls = -1;
 	}
-	else if (notification == STATECHANGE_SERVER)
-	{
+	else if (notification == STATECHANGE_SERVER) {
 		if (!m_tabs)
 			return;
 
