@@ -35,19 +35,19 @@ bool CFileExistsDlg::Create(wxWindow* parent)
 	return true;
 }
 
-void CFileExistsDlg::DisplayFile(bool left, wxString name, int64_t size, CDateTime const& time, wxString const& iconFile)
+void CFileExistsDlg::DisplayFile(bool left, wxString name, int64_t size, fz::datetime const& time, wxString const& iconFile)
 {
 	name = GetPathEllipsis(name, FindWindow(left ? XRCID("ID_FILE1_NAME") : XRCID("ID_FILE2_NAME")));
 	name.Replace(_T("&"), _T("&&"));
 
 	wxString sizeStr = _("Size unknown");
-	if (size != -1) {
+	if (size >= 0) {
 		bool const thousands_separator = COptions::Get()->GetOptionVal(OPTION_SIZE_USETHOUSANDSEP) != 0;
 		sizeStr = CSizeFormat::Format(size, true, CSizeFormat::bytes, thousands_separator, 0);
 	}
 
 	wxString timeStr = _("Date/time unknown");
-	if (time.IsValid())
+	if (!time.empty())
 		timeStr = CTimeFormat::Format(time);
 
 	xrc_call(*this, left ? "ID_FILE1_NAME" : "ID_FILE2_NAME", &wxStaticText::SetLabel, name);
@@ -66,8 +66,8 @@ bool CFileExistsDlg::CreateControls()
 	wxString localFile = m_pNotification->localFile;
 	wxString remoteFile = m_pNotification->remotePath.FormatFilename(m_pNotification->remoteFile);
 
-	DisplayFile(m_pNotification->download, localFile, m_pNotification->localSize.GetValue(), m_pNotification->localTime, m_pNotification->localFile);
-	DisplayFile(!m_pNotification->download, remoteFile, m_pNotification->remoteSize.GetValue(), m_pNotification->remoteTime, m_pNotification->remoteFile);
+	DisplayFile(m_pNotification->download, localFile, m_pNotification->localSize, m_pNotification->localTime, m_pNotification->localFile);
+	DisplayFile(!m_pNotification->download, remoteFile, m_pNotification->remoteSize, m_pNotification->remoteTime, m_pNotification->remoteFile);
 
 	xrc_call(*this, "ID_UPDOWNONLY", &wxCheckBox::SetLabel, m_pNotification->download ? _("A&pply only to downloads") : _("A&pply only to uploads"));
 
@@ -88,7 +88,7 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 #ifdef __WXMSW__
 	SHFILEINFO fileinfo;
 	memset(&fileinfo, 0, sizeof(fileinfo));
-	if (SHGetFileInfo(file, FILE_ATTRIBUTE_NORMAL, &fileinfo, sizeof(fileinfo), SHGFI_ICON | SHGFI_USEFILEATTRIBUTES)) {
+	if (SHGetFileInfo(file.wc_str(), FILE_ATTRIBUTE_NORMAL, &fileinfo, sizeof(fileinfo), SHGFI_ICON | SHGFI_USEFILEATTRIBUTES)) {
 		wxBitmap bmp;
 		bmp.Create(size.x, size.y);
 
@@ -163,7 +163,7 @@ void CFileExistsDlg::LoadIcon(int id, const wxString &file)
 	}
 }
 
-void CFileExistsDlg::OnOK(wxCommandEvent& event)
+void CFileExistsDlg::OnOK(wxCommandEvent&)
 {
 	if (xrc_call(*this, "ID_ACTION1", &wxRadioButton::GetValue))
 		m_action = CFileExistsNotification::overwrite;
@@ -193,7 +193,7 @@ enum CFileExistsNotification::OverwriteAction CFileExistsDlg::GetAction() const
 	return m_action;
 }
 
-void CFileExistsDlg::OnCancel(wxCommandEvent& event)
+void CFileExistsDlg::OnCancel(wxCommandEvent&)
 {
 	m_action = CFileExistsNotification::skip;
 	EndModal(wxID_CANCEL);
