@@ -237,9 +237,6 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 	CreateSystemImageList(16);
 	m_pQueue = pQueue;
 
-	m_sortColumn = 0;
-	m_sortDirection = 0;
-
 	m_hasParent = true;
 
 	m_comparisonIndex = -1;
@@ -285,26 +282,33 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 	CancelLabelEdit();
 
 	if (column != -1) {
-		if (column != m_sortColumn)
-		{
+		if (column != m_sortColumn) {
 			const int oldVisibleColumn = GetColumnVisibleIndex(m_sortColumn);
 			if (oldVisibleColumn != -1)
 				SetHeaderSortIconIndex(oldVisibleColumn, -1);
 		}
 	}
-	else
-		column = m_sortColumn;
+	else {
+		if (m_sortColumn != -1) {
+			column = m_sortColumn;
+		}
+		else {
+			column = 0;
+		}
+	}
 
 	if (direction == -1)
 		direction = m_sortDirection;
 
-	int newVisibleColumn = GetColumnVisibleIndex(column);
-	if (newVisibleColumn == -1) {
-		newVisibleColumn = 0;
-		column = 0;
-	}
 
-	SetHeaderSortIconIndex(newVisibleColumn, direction);
+	if (column != m_sortColumn || direction != m_sortDirection) {
+		int newVisibleColumn = GetColumnVisibleIndex(column);
+		if (newVisibleColumn == -1) {
+			newVisibleColumn = 0;
+			column = 0;
+		}
+		SetHeaderSortIconIndex(newVisibleColumn, direction);
+	}
 
 	// Remember which files are selected
 	bool *selected = 0;
@@ -732,7 +736,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wx
 template<class CFileData> void CFileListCtrl<CFileData>::InitSort(int optionID)
 {
 	wxString sortInfo = COptions::Get()->GetOption(optionID);
-	if( !sortInfo.empty() ) {
+	if (!sortInfo.empty()) {
 		m_sortDirection = sortInfo[0] - '0';
 	}
 	else {
@@ -741,14 +745,16 @@ template<class CFileData> void CFileListCtrl<CFileData>::InitSort(int optionID)
 	if (m_sortDirection < 0 || m_sortDirection > 1)
 		m_sortDirection = 0;
 
-	if (sortInfo.Len() == 3)
-	{
+	if (sortInfo.Len() == 3) {
 		m_sortColumn = sortInfo[2] - '0';
-		if (GetColumnVisibleIndex(m_sortColumn) == -1)
+		if (GetColumnVisibleIndex(m_sortColumn) == -1) {
 			m_sortColumn = 0;
+		}
 	}
 	else
 		m_sortColumn = 0;
+
+	SetHeaderSortIconIndex(GetColumnVisibleIndex(m_sortColumn), m_sortDirection);
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::OnItemSelected(wxListEvent& event)
@@ -1024,3 +1030,28 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnKeyDown(wxKeyEvent& e
 	else
 		event.Skip();
 }
+
+template<class CFileData> void CFileListCtrl<CFileData>::UpdateSelections_ItemsAdded(std::vector<int> const& added_indexes)
+{
+	if (added_indexes.empty()) {
+		return;
+	}
+
+	auto added_index = added_indexes.cbegin();
+	std::deque<bool> selected;
+	// This is O(n) in number of items. I think it's possible to make it O(n) in number of selections
+	for (unsigned int i = *added_index; i < m_indexMapping.size(); ++i) {
+		if (added_index != added_indexes.end() && i == static_cast<unsigned int>(*added_index)) {
+			selected.push_front(false);
+			++added_index;
+		}
+		bool is_selected = GetItemState(i, wxLIST_STATE_SELECTED) != 0;
+		selected.push_back(is_selected);
+
+		bool should_selected = selected.front();
+		selected.pop_front();
+		if (is_selected != should_selected) {
+			SetSelection(i, should_selected);
+		}
+	}
+}
