@@ -1,72 +1,73 @@
 #include <filezilla.h>
 #include "loginmanager.h"
+
+#include "dialogex.h"
 #include "filezillaapp.h"
 
+#include <algorithm>
+
 CLoginManager CLoginManager::m_theLoginManager;
 
-bool CLoginManager::GetPassword(CServer &server, bool silent, wxString name /*=_T("")*/, wxString challenge /*=_T("")*/)
+std::list<CLoginManager::t_passwordcache>::iterator CLoginManager::FindItem(CServer const& server, wxString const& challenge)
 {
-	wxASSERT(!silent || server.GetLogonType() == ASK || server.GetLogonType() == INTERACTIVE);
-	wxASSERT(challenge == _T("") || server.GetLogonType() == INTERACTIVE);
-
-	if (server.GetLogonType() == ASK)
-	{
-		std::list<t_passwordcache>::const_iterator iter;
-		for (iter = m_passwordCache.begin(); iter != m_passwordCache.end(); iter++)
+	return std::find_if(m_passwordCache.begin(), m_passwordCache.end(), [&](t_passwordcache const& item)
 		{
-			if (iter->host != server.GetHost())
-				continue;
-			if (iter->port != server.GetPort())
-				continue;
-			if (iter->user != server.GetUser())
-				continue;
-	
-			server.SetUser(server.GetUser(), iter->password);
+			return item.host == server.GetHost() && item.port == server.GetPort() && item.user == server.GetUser() && item.challenge == challenge;
+		}
+	);
+}
+
+bool CLoginManager::GetPassword(CServer &server, bool silent, wxString const& name, wxString const& challenge, bool canRemember)
+{
+	wxASSERT(server.GetLogonType() != ANONYMOUS);
+
+	if (canRemember) {
+		auto it = FindItem(server, challenge);
+		if (it != m_passwordCache.end()) {
+			server.SetUser(server.GetUser(), it->password);
 			return true;
 		}
 	}
 	if (silent)
 		return false;
 
-	return DisplayDialog(server, name, challenge);
+	return DisplayDialog(server, name, challenge, canRemember);
 }
 
-bool CLoginManager::DisplayDialog(CServer &server, wxString name, wxString challenge)
+bool CLoginManager::DisplayDialog(CServer &server, wxString const& name, wxString challenge, bool canRemember)
 {
-	wxDialog pwdDlg;
-	wxXmlResource::Get()->LoadDialog(&pwdDlg, wxGetApp().GetTopWindow(), _T("ID_ENTERPASSWORD"));
-	if (name == _T(""))
-	{
+	wxDialogEx pwdDlg;
+	if (!pwdDlg.Load(wxGetApp().GetTopWindow(), _T("ID_ENTERPASSWORD"))) {
+		return false;
+	}
+
+	if (name.empty()) {
 		pwdDlg.GetSizer()->Show(XRCCTRL(pwdDlg, "ID_NAMELABEL", wxStaticText), false, true);
 		pwdDlg.GetSizer()->Show(XRCCTRL(pwdDlg, "ID_NAME", wxStaticText), false, true);
 	}
 	else
 		XRCCTRL(pwdDlg, "ID_NAME", wxStaticText)->SetLabel(name);
-	if (challenge == _T(""))
-	{
+	if (challenge.empty()) {
 		pwdDlg.GetSizer()->Show(XRCCTRL(pwdDlg, "ID_CHALLENGELABEL", wxStaticText), false, true);
 		pwdDlg.GetSizer()->Show(XRCCTRL(pwdDlg, "ID_CHALLENGE", wxTextCtrl), false, true);
 	}
-	else
-	{
+	else {
 #ifdef __WXMSW__
 		challenge.Replace(_T("\n"), _T("\r\n"));
 #endif
 		XRCCTRL(pwdDlg, "ID_CHALLENGE", wxTextCtrl)->ChangeValue(challenge);
-		pwdDlg.GetSizer()->Show(XRCCTRL(pwdDlg, "ID_REMEMBER", wxCheckBox), false, true);
+		pwdDlg.GetSizer()->Show(XRCCTRL(pwdDlg, "ID_REMEMBER", wxCheckBox), canRemember, true);
 		XRCCTRL(pwdDlg, "ID_CHALLENGE", wxTextCtrl)->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
 	}
-	XRCCTRL(pwdDlg, "ID_HOST", wxStaticText)->SetLabel(server.FormatHost());
+	XRCCTRL(pwdDlg, "ID_HOST", wxStaticText)->SetLabel(server.Format(ServerFormat::with_optional_port));
 
-	if (server.GetUser() == _T(""))
-	{
+	if (server.GetUser().empty()) {
 		pwdDlg.SetTitle(_("Enter username and password"));
 		XRCCTRL(pwdDlg, "ID_OLD_USER_LABEL", wxStaticText)->Hide();
 		XRCCTRL(pwdDlg, "ID_OLD_USER", wxStaticText)->Hide();
 
 		XRCCTRL(pwdDlg, "ID_HEADER_PASS", wxStaticText)->Hide();
-		if (server.GetLogonType() == INTERACTIVE)
-		{
+		if (server.GetLogonType() == INTERACTIVE) {
 			XRCCTRL(pwdDlg, "ID_PASSWORD_LABEL", wxStaticText)->Hide();
 			XRCCTRL(pwdDlg, "ID_PASSWORD", wxTextCtrl)->Hide();
 			XRCCTRL(pwdDlg, "ID_REMEMBER", wxCheckBox)->Hide();
@@ -77,8 +78,7 @@ bool CLoginManager::DisplayDialog(CServer &server, wxString name, wxString chall
 
 		XRCCTRL(pwdDlg, "ID_NEW_USER", wxTextCtrl)->SetFocus();
 	}
-	else
-	{
+	else {
 		XRCCTRL(pwdDlg, "ID_OLD_USER", wxStaticText)->SetLabel(server.GetUser());
 		XRCCTRL(pwdDlg, "ID_NEW_USER_LABEL", wxStaticText)->Hide();
 		XRCCTRL(pwdDlg, "ID_NEW_USER", wxTextCtrl)->Hide();
@@ -91,17 +91,14 @@ bool CLoginManager::DisplayDialog(CServer &server, wxString name, wxString chall
 	pwdDlg.GetSizer()->SetSizeHints(&pwdDlg);
 
 	wxString user;
-	while (user == _T(""))
-	{
+	while (user.empty()) {
 		if (pwdDlg.ShowModal() != wxID_OK)
 			return false;
 
-		if (server.GetUser() == _T(""))
-		{
+		if (server.GetUser().empty()) {
 			user = XRCCTRL(pwdDlg, "ID_NEW_USER", wxTextCtrl)->GetValue();
-			if (user == _T(""))
-			{
-				wxMessageBox(_("No username given."), _("Invalid input"), wxICON_EXCLAMATION);
+			if (user.empty()) {
+				wxMessageBoxEx(_("No username given."), _("Invalid input"), wxICON_EXCLAMATION);
 				continue;
 			}
 		}
@@ -111,34 +108,38 @@ bool CLoginManager::DisplayDialog(CServer &server, wxString name, wxString chall
 
 	server.SetUser(user, XRCCTRL(pwdDlg, "ID_PASSWORD", wxTextCtrl)->GetValue());
 
-	if (server.GetLogonType() == ASK && XRCCTRL(pwdDlg, "ID_REMEMBER", wxCheckBox)->GetValue())
-	{
-		t_passwordcache entry;
-		entry.host = server.GetHost();
-		entry.port = server.GetPort();
-		entry.user = server.GetUser();
-		entry.password = server.GetPass();
-		m_passwordCache.push_back(entry);
+	if (canRemember) {
+		RememberPassword(server, challenge);
 	}
 
 	return true;
 }
 
-void CLoginManager::CachedPasswordFailed(const CServer& server)
+void CLoginManager::CachedPasswordFailed(const CServer& server, wxString const& challenge)
 {
-	if (server.GetLogonType() != ASK)
+	auto it = FindItem(server, challenge);
+	if (it != m_passwordCache.end()) {
+		m_passwordCache.erase(it);
+	}
+}
+
+void CLoginManager::RememberPassword(CServer & server, wxString const& challenge)
+{
+	if (server.GetLogonType() == ANONYMOUS) {
 		return;
-	
-	for (std::list<t_passwordcache>::iterator iter = m_passwordCache.begin(); iter != m_passwordCache.end(); iter++)
-	{
-		if (iter->host != server.GetHost())
-			continue;
-		if (iter->port != server.GetPort())
-			continue;
-		if (iter->user != server.GetUser())
-			continue;
-	
-		m_passwordCache.erase(iter);
-		return;
+	}
+
+	auto it = FindItem(server, challenge);
+	if (it != m_passwordCache.end()) {
+		it->password = server.GetPass();
+	}
+	else {
+		t_passwordcache entry;
+		entry.host = server.GetHost();
+		entry.port = server.GetPort();
+		entry.user = server.GetUser();
+		entry.password = server.GetPass();
+		entry.challenge = challenge;
+		m_passwordCache.push_back(entry);
 	}
 }
