@@ -391,7 +391,7 @@ const CServer& CServerItem::GetServer() const
 
 wxString CServerItem::GetName() const
 {
-	return m_server.FormatServer();
+	return m_server.Format(ServerFormat::with_user_and_optional_port);
 }
 
 void CServerItem::AddChild(CQueueItem* pItem)
