@@ -153,8 +153,8 @@ public:
 	sqlite3_stmt* PrepareInsertStatement(const wxString& name, const _column*, unsigned int count);
 
 	bool SaveServer(const CServerItem& item);
-	bool SaveFile(wxLongLong server, const CFileItem& item);
-	bool SaveDirectory(wxLongLong server, const CFolderItem& item);
+	bool SaveFile(const CFileItem& item);
+	bool SaveDirectory(const CFolderItem& item);
 
 	int64_t SaveLocalPath(const CLocalPath& path);
 	int64_t SaveRemotePath(const CServerPath& path);
@@ -221,7 +221,7 @@ void CQueueStorage::Impl::ReadLocalPaths()
 			int64_t id = GetColumnInt64(selectLocalPathQuery_, path_table_column_names::id);
 			wxString localPathRaw = GetColumnText(selectLocalPathQuery_, path_table_column_names::path);
 			CLocalPath localPath;
-			if (id > 0 && !localPathRaw.empty() && localPath.SetPath(localPathRaw))
+			if (id > 0 && !localPathRaw.empty() && localPath.SetPath(localPathRaw.ToStdWstring()))
 				reverseLocalPaths_[id] = localPath;
 		}
 	}
@@ -261,7 +261,7 @@ const CLocalPath& CQueueStorage::Impl::GetLocalPath(int64_t id) const
 	if (it != reverseLocalPaths_.end())
 		return it->second;
 
-	static const CLocalPath empty;
+	static CLocalPath const empty{};
 	return empty;
 }
 
@@ -272,7 +272,7 @@ const CServerPath& CQueueStorage::Impl::GetRemotePath(int64_t id) const
 	if (it != reverseRemotePaths_.end())
 		return it->second;
 
-	static const CServerPath empty;
+	static CServerPath const empty{};
 	return empty;
 }
 
@@ -591,20 +591,16 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 	Bind(insertServerQuery_, server_table_column_names::type, static_cast<int>(server.GetType()));
 
 	enum LogonType logonType = server.GetLogonType();
-	if (server.GetLogonType() != ANONYMOUS)
-	{
+	if (server.GetLogonType() != ANONYMOUS) {
 		Bind(insertServerQuery_, server_table_column_names::user, server.GetUser());
 
-		if (server.GetLogonType() == NORMAL || server.GetLogonType() == ACCOUNT)
-		{
-			if (kiosk_mode)
-			{
+		if (server.GetLogonType() == NORMAL || server.GetLogonType() == ACCOUNT) {
+			if (kiosk_mode) {
 				logonType = ASK;
 				BindNull(insertServerQuery_, server_table_column_names::password);
 				BindNull(insertServerQuery_, server_table_column_names::account);
 			}
-			else
-			{
+			else {
 				Bind(insertServerQuery_, server_table_column_names::password, server.GetPass());
 
 				if (server.GetLogonType() == ACCOUNT)
@@ -613,14 +609,12 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 					BindNull(insertServerQuery_, server_table_column_names::account);
 			}
 		}
-		else
-		{
+		else {
 			BindNull(insertServerQuery_, server_table_column_names::password);
 			BindNull(insertServerQuery_, server_table_column_names::account);
 		}
 	}
-	else
-	{
+	else {
 			BindNull(insertServerQuery_, server_table_column_names::user);
 			BindNull(insertServerQuery_, server_table_column_names::password);
 			BindNull(insertServerQuery_, server_table_column_names::account);
@@ -688,26 +682,24 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 	sqlite3_reset(insertServerQuery_);
 
 	bool ret = res == SQLITE_DONE;
-	if (ret)
-	{
+	if (ret) {
 		sqlite3_int64 serverId = sqlite3_last_insert_rowid(db_);
 		Bind(insertFileQuery_, file_table_column_names::server, static_cast<int64_t>(serverId));
 
 		const std::vector<CQueueItem*>& children = item.GetChildren();
-		for (std::vector<CQueueItem*>::const_iterator it = children.begin() + item.GetRemovedAtFront(); it != children.end(); ++it)
-		{
-			CQueueItem* item = *it;
-			if (item->GetType() == QueueItemType::File)
-				ret &= SaveFile(serverId, *static_cast<CFileItem*>(item));
-			else if (item->GetType() == QueueItemType::Folder)
-				ret &= SaveDirectory(serverId, *static_cast<CFolderItem*>(item));
+		for (std::vector<CQueueItem*>::const_iterator it = children.begin() + item.GetRemovedAtFront(); it != children.end(); ++it) {
+			CQueueItem & childItem = **it;
+			if (childItem.GetType() == QueueItemType::File)
+				ret &= SaveFile(static_cast<CFileItem&>(childItem));
+			else if (childItem.GetType() == QueueItemType::Folder)
+				ret &= SaveDirectory(static_cast<CFolderItem&>(childItem));
 		}
 	}
 	return ret;
 }
 
 
-bool CQueueStorage::Impl::SaveFile(wxLongLong server, const CFileItem& file)
+bool CQueueStorage::Impl::SaveFile(const CFileItem& file)
 {
 	if (file.m_edit != CEditHandler::none)
 		return true;
@@ -755,7 +747,7 @@ bool CQueueStorage::Impl::SaveFile(wxLongLong server, const CFileItem& file)
 }
 
 
-bool CQueueStorage::Impl::SaveDirectory(wxLongLong server, const CFolderItem& directory)
+bool CQueueStorage::Impl::SaveDirectory(const CFolderItem& directory)
 {
 	if (directory.Download())
 		BindNull(insertFileQuery_, file_table_column_names::source_file);
