@@ -137,7 +137,7 @@ size_t CContextManager::HandlerCount(t_statechange_notifications notification) c
 	return m_handlers[notification].size();
 }
 
-void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2)
+void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications notification, wxString const& data, void const* data2)
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
@@ -158,13 +158,13 @@ CState* CContextManager::GetCurrentContext()
 	return m_contexts[m_current_context];
 }
 
-void CContextManager::NotifyAllHandlers(t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+void CContextManager::NotifyAllHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
 {
 	for (unsigned int i = 0; i < m_contexts.size(); i++)
 		m_contexts[i]->NotifyHandlers(notification, data, data2);
 }
 
-void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
+void CContextManager::NotifyGlobalHandlers(t_statechange_notifications notification, wxString const& data, void const* data2)
 {
 	auto const& handlers = m_handlers[notification];
 	for (auto const& handler : handlers) {
@@ -185,7 +185,7 @@ CState::CState(CMainFrame &mainFrame)
 	m_sync_browse.is_changing = false;
 	m_sync_browse.compare = false;
 
-	m_localDir.SetPath(CLocalPath::path_separator);
+	m_localDir.SetPath(std::wstring(1, CLocalPath::path_separator));
 }
 
 CState::~CState()
@@ -422,13 +422,13 @@ void CState::RefreshLocalFile(wxString file)
 
 void CState::LocalDirCreated(const CLocalPath& path)
 {
-	if (!path.IsSubdirOf(m_localDir))
+	if (!path.IsSubdirOf(m_localDir)) {
 		return;
+	}
 
-	wxString next_segment = path.GetPath().Mid(m_localDir.GetPath().Len());
+	wxString next_segment = path.GetPath().substr(m_localDir.GetPath().size());
 	int pos = next_segment.Find(CLocalPath::path_separator);
-	if (pos <= 0)
-	{
+	if (pos <= 0) {
 		// Shouldn't ever come true
 		return;
 	}
@@ -468,10 +468,11 @@ void CState::SetServer(const CServer* server, CServerPath const& path)
 		m_pServer = new CServer(*server);
 
 		const wxString& name = server->GetName();
-		if (!name.empty())
-			m_title = name + _T(" - ") + server->FormatServer();
-		else
-			m_title = server->FormatServer();
+		m_title.clear();
+		if (!name.empty()) {
+			m_title = name + _T(" - ");
+		}
+		m_title += server->Format(ServerFormat::with_user_and_optional_port);
 	}
 	else {
 		m_pServer = 0;
@@ -715,9 +716,9 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 	}
 	*p = 0; // End of list
 
-	wxChar* to = new wxChar[path.GetPath().Len() + 2];
+	wxChar* to = new wxChar[path.GetPath().size() + 2];
 	wxStrcpy(to, path.GetPath());
-	to[path.GetPath().Len() + 1] = 0; // End of list
+	to[path.GetPath().size() + 1] = 0; // End of list
 
 	SHFILEOPSTRUCT op = {0};
 	op.pFrom = from;
@@ -886,7 +887,7 @@ bool CState::IsRemoteConnected() const
 
 bool CState::IsRemoteIdle(bool ignore_recursive) const
 {
-	if (!ignore_recursive && m_pRemoteRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
+	if (!ignore_recursive && m_pRemoteRecursiveOperation && m_pRemoteRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
 		return false;
 
 	if (!m_pCommandQueue)
@@ -895,6 +896,14 @@ bool CState::IsRemoteIdle(bool ignore_recursive) const
 	return m_pCommandQueue->Idle(ignore_recursive ? CCommandQueue::normal : CCommandQueue::any);
 }
 
+bool CState::IsLocalIdle(bool ignore_recursive) const
+{
+	if (!ignore_recursive && m_pLocalRecursiveOperation && m_pLocalRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_none)
+		return false;
+
+	return true;
+}
+
 void CState::ListingFailed(int)
 {
 	if (m_sync_browse.is_changing && !m_sync_browse.target_path.empty()) {
@@ -909,7 +918,6 @@ void CState::ListingFailed(int)
 		xrc_call(dlg, "ID_SYNCBROWSE_DISABLE", &wxRadioButton::SetLabel, _("&Disable synchronized browsing and continue changing the local directory"));
 		dlg.GetSizer()->Fit(&dlg);
 		if (dlg.ShowModal() == wxID_OK) {
-
 			if (xrc_call(dlg, "ID_SYNCBROWSE_CREATE", &wxRadioButton::GetValue)) {
 				m_pCommandQueue->ProcessCommand(new CMkdirCommand(m_sync_browse.target_path));
 				m_pCommandQueue->ProcessCommand(new CListCommand(m_sync_browse.target_path));
