@@ -93,7 +93,7 @@ void CViewHeader::OnSize(wxSizeEvent&)
 	rect.SetWidth(rect.GetWidth() - m_cbOffset + 2);
 	rect.SetX(m_cbOffset);
 	rect.Deflate(0, border_offset / 2);
-	rect.SetWidth(rect.GetWidth() - border_offset / 2);
+	rect.SetWidth(std::max(0, rect.GetWidth() - border_offset / 2));
 	if (m_pComboBox) {
 		m_pComboBox->SetSize(rect);
 	}
@@ -330,10 +330,10 @@ EVT_COMMAND(wxID_ANY, fzEVT_LOCALVIEWHEADERSETTEXTSEL, CLocalViewHeader::OnSelec
 #endif
 END_EVENT_TABLE()
 
-CLocalViewHeader::CLocalViewHeader(wxWindow* pParent, CState* pState)
-	: CViewHeader(pParent, _("Local site:")), CStateEventHandler(pState)
+CLocalViewHeader::CLocalViewHeader(wxWindow* pParent, CState& state)
+	: CViewHeader(pParent, _("Local site:")), CStateEventHandler(state)
 {
-	pState->RegisterHandler(this, STATECHANGE_LOCAL_DIR);
+	state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
 }
 
 void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
@@ -345,14 +345,12 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 #endif
 
 	wxString str = m_pComboBox->GetValue();
-	if (str.empty() || str.Right(1) == _T("/"))
-	{
+	if (str.empty() || str.Right(1) == _T("/")) {
 		m_oldValue = str;
 		return;
 	}
 #ifdef __WXMSW__
-	if (str.Right(1) == _T("\\"))
-	{
+	if (str.Right(1) == _T("\\")) {
 		m_oldValue = str;
 		return;
 	}
@@ -361,25 +359,21 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 	if (str == m_oldValue)
 		return;
 
-	if (str.Left(m_oldValue.Length()) != m_oldValue)
-	{
+	if (str.Left(m_oldValue.Length()) != m_oldValue) {
 		m_oldValue = str;
 		return;
 	}
 
 #ifdef __WXMSW__
-	if (str.Left(2) == _T("\\\\"))
-	{
+	if (str.Left(2) == _T("\\\\")) {
 		int pos = str.Mid(2).Find('\\');
-		if (pos == -1)
-		{
+		if (pos == -1) {
 			// Partial UNC path, no full server yet, skip further processing
 			return;
 		}
 
 		pos = str.Mid(pos + 3).Find('\\');
-		if (pos == -1)
-		{
+		if (pos == -1) {
 			// Partial UNC path, no full share yet, skip further processing
 			return;
 		}
@@ -387,8 +381,7 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 #endif
 
 	wxFileName fn(str);
-	if (!fn.IsOk())
-	{
+	if (!fn.IsOk()) {
 		m_oldValue = str;
 		return;
 	}
@@ -397,16 +390,13 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 	const wxString path = fn.GetPath();
 
 	wxDir dir;
-	if (name.empty() || path.empty())
-	{
+	if (name.empty() || path.empty()) {
 		m_oldValue = str;
 		return;
 	}
-	else
-	{
+	else {
 		wxLogNull log;
-		if (!dir.Open(path) || !dir.IsOpened())
-		{
+		if (!dir.Open(path) || !dir.IsOpened()) {
 			m_oldValue = str;
 			return;
 		}
@@ -415,16 +405,14 @@ void CLocalViewHeader::OnTextChanged(wxCommandEvent&)
 
 	{
 		wxLogNull noLog;
-		if (!dir.GetFirst(&found, name + _T("*"), wxDIR_DIRS))
-		{
+		if (!dir.GetFirst(&found, name + _T("*"), wxDIR_DIRS)) {
 			m_oldValue = str;
 			return;
 		}
 	}
 
 	wxString tmp;
-	if (dir.GetNext(&tmp))
-	{
+	if (dir.GetNext(&tmp)) {
 		m_oldValue = str;
 		return;
 	}
@@ -478,9 +466,8 @@ void CLocalViewHeader::OnSelectionChanged(wxCommandEvent& event)
 	if (dir.empty())
 		return;
 
-	if (!wxDir::Exists(dir))
-	{
-		const wxString& current = m_pState->GetLocalDir().GetPath();
+	if (!wxDir::Exists(dir)) {
+		const wxString& current = m_state.GetLocalDir().GetPath();
 		int item = m_pComboBox->FindString(current, true);
 		if (item != wxNOT_FOUND)
 			m_pComboBox->SetSelection(item);
@@ -489,7 +476,7 @@ void CLocalViewHeader::OnSelectionChanged(wxCommandEvent& event)
 		return;
 	}
 
-	m_pState->SetLocalDir(dir);
+	m_state.SetLocalDir(dir);
 }
 
 void CLocalViewHeader::OnTextEnter(wxCommandEvent&)
@@ -501,17 +488,16 @@ void CLocalViewHeader::OnTextEnter(wxCommandEvent&)
 	wxString dir = m_pComboBox->GetValue();
 
 	wxString error;
-	if (!m_pState->SetLocalDir(dir, &error))
-	{
+	if (!m_state.SetLocalDir(dir, &error)) {
 		if (!error.empty())
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 		else
 			wxBell();
-		m_pComboBox->SetValue(m_pState->GetLocalDir().GetPath());
+		m_pComboBox->SetValue(m_state.GetLocalDir().GetPath());
 	}
 }
 
-void CLocalViewHeader::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+void CLocalViewHeader::OnStateChange(t_statechange_notifications notification, const wxString&, const void*)
 {
 	wxASSERT(notification == STATECHANGE_LOCAL_DIR);
 	(void)notification;
@@ -520,7 +506,7 @@ void CLocalViewHeader::OnStateChange(CState* pState, enum t_statechange_notifica
 	m_autoCompletionText = _T("");
 #endif
 
-	wxString dir = pState->GetLocalDir().GetPath();
+	wxString dir = m_state.GetLocalDir().GetPath();
 	AddRecentDirectory(dir);
 }
 
@@ -529,29 +515,26 @@ EVT_TEXT_ENTER(wxID_ANY, CRemoteViewHeader::OnTextEnter)
 EVT_COMBOBOX(wxID_ANY, CRemoteViewHeader::OnSelectionChanged)
 END_EVENT_TABLE()
 
-CRemoteViewHeader::CRemoteViewHeader(wxWindow* pParent, CState* pState)
-	: CViewHeader(pParent, _("Remote site:")), CStateEventHandler(pState)
+CRemoteViewHeader::CRemoteViewHeader(wxWindow* pParent, CState& state)
+	: CViewHeader(pParent, _("Remote site:")), CStateEventHandler(state)
 {
-	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
+	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	Disable();
 }
 
-void CRemoteViewHeader::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
+void CRemoteViewHeader::OnStateChange(t_statechange_notifications notification, const wxString&, const void*)
 {
 	wxASSERT(notification == STATECHANGE_REMOTE_DIR);
 	(void)notification;
 
-	m_path = pState->GetRemotePath();
-	if (m_path.empty())
-	{
+	m_path = m_state.GetRemotePath();
+	if (m_path.empty()) {
 		m_pComboBox->SetValue(_T(""));
 		Disable();
 	}
-	else
-	{
-		const CServer* const pServer = pState->GetServer();
-		if (pServer && *pServer != m_lastServer)
-		{
+	else {
+		const CServer* const pServer = m_state.GetServer();
+		if (pServer && *pServer != m_lastServer) {
 			m_pComboBox->Clear();
 			m_recentDirectories.clear();
 			m_sortedRecentDirectories.clear();
@@ -569,19 +552,17 @@ void CRemoteViewHeader::OnTextEnter(wxCommandEvent&)
 {
 	CServerPath path = m_path;
 	wxString value = m_pComboBox->GetValue();
-	if (value.empty() || !path.ChangePath(value))
-	{
+	if (value.empty() || !path.ChangePath(value)) {
 		wxBell();
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_state.IsRemoteIdle(true)) {
 		wxBell();
 		return;
 	}
 
-	m_pState->ChangeRemoteDir(path);
+	m_state.ChangeRemoteDir(path);
 }
 
 void CRemoteViewHeader::OnSelectionChanged(wxCommandEvent& event)
@@ -591,17 +572,15 @@ void CRemoteViewHeader::OnSelectionChanged(wxCommandEvent& event)
 		return;
 
 	CServerPath path = m_path;
-	if (!path.SetPath(dir))
-	{
+	if (!path.SetPath(dir)) {
 		wxBell();
 		return;
 	}
 
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_state.IsRemoteIdle(true)) {
 		wxBell();
 		return;
 	}
 
-	m_pState->ChangeRemoteDir(path);
+	m_state.ChangeRemoteDir(path);
 }
