@@ -6,6 +6,8 @@
 #include "buildinfo.h"
 #include "Options.h"
 
+#include <libfilezilla/local_filesys.hpp>
+
 #include <wx/statbox.h>
 #include <wx/wizard.h>
 
@@ -34,18 +36,12 @@ bool CWrapEngine::CanWrapBefore(const wxChar& c)
 {
 	// Check if this is a punctuation character, we're not allowed
 	// to wrap before such a character
-	const wxChar* p = m_noWrapChars;
-	while (*p)
-	{
-		if (*p == c)
-			break;
-
-		p++;
+	for (auto p = m_noWrapChars; p && *p; ++p) {
+		if (*p == c) {
+			return false;
+		}
 	}
-	if (!*p)
-		return true;
-
-	return false;
+	return true;
 }
 
 bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned long maxLength)
@@ -342,7 +338,7 @@ bool CWrapEngine::WrapText(wxWindow* parent, wxString& text, unsigned long maxLe
 
 bool CWrapEngine::WrapText(wxWindow* parent, int id, unsigned long maxLength)
 {
-	wxStaticText* pText = wxDynamicCast(parent->FindWindow(id), wxStaticText);
+	wxStaticText* pText = dynamic_cast<wxStaticText*>(parent->FindWindow(id));
 	if (!pText)
 		return false;
 
@@ -403,7 +399,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 		wxWindow* window;
 		wxSizer* subSizer = 0;
 		if ((window = item->GetWindow())) {
-			wxStaticText* text = wxDynamicCast(window, wxStaticText);
+			wxStaticText* text = dynamic_cast<wxStaticText*>(window);
 			if (text) {
 #ifdef __WXMAC__
 				const int offset = 3;
@@ -433,7 +429,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 				continue;
 			}
 
-			wxNotebook* book = wxDynamicCast(window, wxNotebook);
+			wxNotebook* book = dynamic_cast<wxNotebook*>(window);
 			if (book) {
 				int maxPageWidth = 0;
 				for (unsigned int j = 0; j < book->GetPageCount(); ++j) {
@@ -457,8 +453,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 				continue;
 			}
 
-			if (wxDynamicCast(window, wxCheckBox) || wxDynamicCast(window, wxRadioButton) || wxDynamicCast(window, wxChoice))
-			{
+			if (dynamic_cast<wxCheckBox*>(window) || dynamic_cast<wxRadioButton*>(window) || dynamic_cast<wxChoice*>(window)) {
 #if WRAPDEBUG >= 3
 				plvl printf("Leave: WrapRecursive on unshrinkable control failed: %s\n",
 					static_cast<char const*>(wxString(window->GetClassInfo()->GetClassName())));
@@ -476,8 +471,7 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 
 			// Add border of static box sizer
 			wxStaticBoxSizer* sboxSizer;
-			if ((sboxSizer = wxDynamicCast(subSizer, wxStaticBoxSizer)))
-			{
+			if ((sboxSizer = dynamic_cast<wxStaticBoxSizer*>(subSizer))) {
 				int top, other;
 				sboxSizer->GetStaticBox()->GetBordersForSizer(&top, &other);
 				subBorder += other * 2;
@@ -501,8 +495,8 @@ int CWrapEngine::WrapRecursive(wxWindow* wnd, wxSizer* sizer, int max)
 		}
 	}
 
-	wxStaticBoxSizer* sboxSizer = wxDynamicCast(sizer, wxStaticBoxSizer);
-	if( sboxSizer ) {
+	wxStaticBoxSizer* sboxSizer = dynamic_cast<wxStaticBoxSizer*>(sizer);
+	if (sboxSizer) {
 #ifdef __WXGTK3__
 		gtk_widget_set_size_request(sboxSizer->GetStaticBox()->GetHandle(), -1, -1);
 		GtkRequisition req;
@@ -787,22 +781,18 @@ bool CWrapEngine::UnwrapRecursive(wxWindow* wnd, wxSizer* sizer)
 
 		wxWindow* window;
 		wxSizer* subSizer;
-		if ((window = item->GetWindow()))
-		{
-			wxStaticText* text = wxDynamicCast(window, wxStaticText);
-			if (text)
-			{
+		if ((window = item->GetWindow())) {
+			wxStaticText* text = dynamic_cast<wxStaticText*>(window);
+			if (text) {
 				wxString unwrapped = UnwrapText(text->GetLabel());
 				text->SetLabel(unwrapped);
 
 				continue;
 			}
 
-			wxNotebook* book = wxDynamicCast(window, wxNotebook);
-			if (book)
-			{
-				for (unsigned int j = 0; j < book->GetPageCount(); ++j)
-				{
+			wxNotebook* book = dynamic_cast<wxNotebook*>(window);
+			if (book) {
+				for (unsigned int j = 0; j < book->GetPageCount(); ++j) {
 					wxNotebookPage* page = book->GetPage(j);
 					UnwrapRecursive(wnd, page->GetSizer());
 				}
@@ -831,9 +821,9 @@ int CWrapEngine::GetWidthFromCache(const char* name)
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
 
 	CXmlFile xml(wxGetApp().GetSettingsFile(_T("layout")));
-	TiXmlElement* root = xml.Load();
-	TiXmlElement* pElement = root ? root->FirstChildElement("Layout") : 0;
-	if (!pElement) {
+	auto root = xml.Load();
+	auto  element = root.child("Layout");
+	if (!element) {
 		return 0;
 	}
 
@@ -841,17 +831,17 @@ int CWrapEngine::GetWidthFromCache(const char* name)
 	if (language.empty())
 		language = _T("default");
 
-	TiXmlElement* pLanguage = FindElementWithAttribute(pElement, "Language", "id", language.mb_str());
-	if (!pLanguage) {
+	auto xLanguage = FindElementWithAttribute(element, "Language", "id", language.mb_str());
+	if (!xLanguage) {
 		return 0;
 	}
 
-	TiXmlElement* pDialog = FindElementWithAttribute(pLanguage, "Dialog", "name", name);
-	if (!pDialog) {
+	auto dialog = FindElementWithAttribute(xLanguage, "Dialog", "name", name);
+	if (!dialog) {
 		return 0;
 	}
 
-	int value = GetAttributeInt(pDialog, "width");
+	int value = GetAttributeInt(dialog, "width");
 
 	return value;
 }
@@ -869,9 +859,9 @@ void CWrapEngine::SetWidthToCache(const char* name, int width)
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
 
 	CXmlFile xml(wxGetApp().GetSettingsFile(_T("layout")));
-	TiXmlElement* root = xml.Load();
-	TiXmlElement* pElement = root ? root->FirstChildElement("Layout") : 0;
-	if (!pElement) {
+	auto root = xml.Load();
+	auto element = root.child("Layout");
+	if (!element) {
 		return;
 	}
 
@@ -879,18 +869,18 @@ void CWrapEngine::SetWidthToCache(const char* name, int width)
 	if (language.empty())
 		language = _T("default");
 
-	TiXmlElement* pLanguage = FindElementWithAttribute(pElement, "Language", "id", language.mb_str());
-	if (!pLanguage) {
+	auto xLanguage = FindElementWithAttribute(element, "Language", "id", language.mb_str());
+	if (!xLanguage) {
 		return;
 	}
 
-	TiXmlElement* pDialog = FindElementWithAttribute(pLanguage, "Dialog", "name", name);
-	if (!pDialog) {
-		pDialog = pLanguage->LinkEndChild(new TiXmlElement("Dialog"))->ToElement();
-		pDialog->SetAttribute("name", name);
+	auto dialog = FindElementWithAttribute(xLanguage, "Dialog", "name", name);
+	if (!dialog) {
+		dialog = xLanguage.append_child("Dialog");
+		SetTextAttribute(dialog, "name", name);
 	}
 
-	pDialog->SetAttribute("width", width);
+	SetAttributeInt(dialog, "width", width);
 	xml.Save(false);
 }
 
@@ -936,10 +926,9 @@ bool CWrapEngine::LoadCache()
 	CInterProcessMutex mutex(MUTEX_LAYOUT);
 
 	CXmlFile xml(wxGetApp().GetSettingsFile(_T("layout")));
-	TiXmlElement* pDocument = xml.Load();
+	auto document = xml.Load();
 
-	if (!pDocument)
-	{
+	if (!document) {
 		m_use_cache = false;
 		wxMessageBoxEx(xml.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
@@ -948,30 +937,28 @@ bool CWrapEngine::LoadCache()
 
 	bool cacheValid = true;
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Layout");
-	if (!pElement)
-		pElement = pDocument->LinkEndChild(new TiXmlElement("Layout"))->ToElement();
+	auto layoutElement = document.child("Layout");
+	if (!layoutElement)
+		layoutElement = document.append_child("Layout");
 
 	const wxString buildDate = CBuildInfo::GetBuildDateString();
-	if (GetTextAttribute(pElement, "Builddate") != buildDate)
-	{
+	if (GetTextAttribute(layoutElement, "Builddate") != buildDate) {
 		cacheValid = false;
-		SetTextAttribute(pElement, "Builddate", buildDate);
+		SetTextAttribute(layoutElement, "Builddate", buildDate);
 	}
 
 	const wxString buildTime = CBuildInfo::GetBuildTimeString();
-	if (GetTextAttribute(pElement, "Buildtime") != buildTime)
-	{
+	if (GetTextAttribute(layoutElement, "Buildtime") != buildTime) {
 		cacheValid = false;
-		SetTextAttribute(pElement, "Buildtime", buildTime);
+		SetTextAttribute(layoutElement, "Buildtime", buildTime);
 	}
 
 	// Enumerate resource file names
 	// -----------------------------
 
-	TiXmlElement* pResources = pElement->FirstChildElement("Resources");
-	if (!pResources)
-		pResources = pElement->LinkEndChild(new TiXmlElement("Resources"))->ToElement();
+	auto resources = layoutElement.child("Resources");
+	if (!resources)
+		resources = layoutElement.append_child("Resources");
 
 	CLocalPath resourceDir = wxGetApp().GetResourceDir();
 	resourceDir.AddSegment(_T("xrc"));
@@ -980,43 +967,35 @@ bool CWrapEngine::LoadCache()
 	wxLogNull log;
 
 	wxString xrc;
-	for (bool found = dir.GetFirst(&xrc, _T("*.xrc")); found; found = dir.GetNext(&xrc))
-	{
+	for (bool found = dir.GetFirst(&xrc, _T("*.xrc")); found; found = dir.GetNext(&xrc)) {
 		if (!wxFileName::FileExists(resourceDir.GetPath() + xrc))
 			continue;
 
-		wxFileName fn(resourceDir.GetPath() + xrc);
-		wxDateTime date = fn.GetModificationTime();
-		wxLongLong ticks = date.GetTicks();
+		fz::datetime const date = fz::local_filesys::get_modification_time(fz::to_native(resourceDir.GetPath() + xrc));
+		wxString const ticks = std::to_wstring(date.get_time_t());
 
-		TiXmlElement* resourceElement = FindElementWithAttribute(pResources, "xrc", "file", xrc.mb_str());
-		if (!resourceElement)
-		{
-			resourceElement = pResources->LinkEndChild(new TiXmlElement("xrc"))->ToElement();
-			resourceElement->SetAttribute("file", xrc.mb_str());
-			resourceElement->SetAttribute("date", ticks.ToString().mb_str());
+		auto resourceElement = FindElementWithAttribute(resources, "xrc", "file", xrc.mb_str());
+		if (!resourceElement) {
+			resourceElement = resources.append_child("xrc");
+			SetTextAttribute(resourceElement, "file", xrc);
+			SetTextAttribute(resourceElement, "date", ticks);
 			cacheValid = false;
 		}
-		else
-		{
-			const char* xrcNodeDate = resourceElement->Attribute("date");
-			if (!xrcNodeDate || strcmp(xrcNodeDate, ticks.ToString().mb_str()))
-			{
+		else {
+			wxString xrcNodeDate = GetTextAttribute(resourceElement, "date");
+			if (xrcNodeDate.empty() || xrcNodeDate != ticks) {
 				cacheValid = false;
-
-				resourceElement->SetAttribute("date", ticks.ToString().mb_str());
+				SetTextAttribute(resourceElement, "date", ticks);
 			}
 		}
 	}
 
-	if (!cacheValid)
-	{
+	if (!cacheValid) {
 		// Clear all languages
-		TiXmlElement* pLanguage = pElement->FirstChildElement("Language");
-		while (pLanguage)
-		{
-			pElement->RemoveChild(pLanguage);
-			pLanguage = pElement->FirstChildElement("Language");
+		auto language = layoutElement.child("Language");
+		while (language) {
+			layoutElement.remove_child(language);
+			language = layoutElement.child("Language");
 		}
 	}
 
@@ -1025,11 +1004,10 @@ bool CWrapEngine::LoadCache()
 	if (language.empty())
 		language = _T("default");
 
-	TiXmlElement* languageElement = FindElementWithAttribute(pElement, "Language", "id", language.mb_str());
-	if (!languageElement)
-	{
-		languageElement = pElement->LinkEndChild(new TiXmlElement("Language"))->ToElement();
-		languageElement->SetAttribute("id", language.mb_str());
+	auto languageElement = FindElementWithAttribute(layoutElement, "Language", "id", language.mb_str());
+	if (!languageElement) {
+		languageElement = layoutElement.append_child("Language");
+		SetTextAttribute(languageElement, "id", language.mb_str());
 	}
 
 	// Get static text font and measure sample text
@@ -1040,25 +1018,24 @@ bool CWrapEngine::LoadCache()
 	wxFont font = pText->GetFont();
 	wxString fontDesc = font.GetNativeFontInfoDesc();
 
-	TiXmlElement* pFontElement = languageElement->FirstChildElement("Font");
-	if (!pFontElement)
-		pFontElement = languageElement->LinkEndChild(new TiXmlElement("Font"))->ToElement();
+	auto fontElement = languageElement.child("Font");
+	if (!fontElement)
+		fontElement = languageElement.append_child("Font");
 
-	if (GetTextAttribute(pFontElement, "font") != fontDesc)
-	{
-		SetTextAttribute(pFontElement, "font", fontDesc);
+	if (GetTextAttribute(fontElement, "font") != fontDesc) {
+		SetTextAttribute(fontElement, "font", fontDesc);
 		cacheValid = false;
 	}
 
 	int width, height;
 	pText->GetTextExtent(_T("Just some test string we are measuring. If width or heigh differ from the recorded values, invalidate cache. 1234567890MMWWII"), &width, &height);
 
-	if (GetAttributeInt(pFontElement, "width") != width ||
-		GetAttributeInt(pFontElement, "height") != height)
+	if (GetAttributeInt(fontElement, "width") != width ||
+		GetAttributeInt(fontElement, "height") != height)
 	{
 		cacheValid = false;
-		SetAttributeInt(pFontElement, "width", width);
-		SetAttributeInt(pFontElement, "height", height);
+		SetAttributeInt(fontElement, "width", width);
+		SetAttributeInt(fontElement, "height", height);
 	}
 
 	pFrame->Destroy();
@@ -1067,30 +1044,25 @@ bool CWrapEngine::LoadCache()
 	CLocalPath const localesDir = wxGetApp().GetLocalesDir();
 	wxString name = GetLocaleFile(localesDir.GetPath(), language);
 
-	if (!name.empty())
-	{
-		wxFileName fn(localesDir.GetPath() + name + _T("/filezilla.mo"));
-		wxDateTime date = fn.GetModificationTime();
-		wxLongLong ticks = date.GetTicks();
+	if (!name.empty()) {
+		fz::datetime const date = fz::local_filesys::get_modification_time(fz::to_native(localesDir.GetPath() + name + _T("/filezilla.mo")));
+		wxString const ticks = std::to_wstring(date.get_time_t());
 
-		const char* languageNodeDate = languageElement->Attribute("date");
-		if (!languageNodeDate || strcmp(languageNodeDate, ticks.ToString().mb_str()))
-		{
-			languageElement->SetAttribute("date", ticks.ToString().mb_str());
+		wxString languageNodeDate = GetTextAttribute(languageElement, "date");
+		if (languageNodeDate.empty() || languageNodeDate != ticks) {
+			SetTextAttribute(languageElement, "date", ticks);
 			cacheValid = false;
 		}
 	}
 	else
-		languageElement->SetAttribute("date", "");
-	if (!cacheValid)
-	{
-		TiXmlElement* dialog;
-		while ((dialog = languageElement->FirstChildElement("Dialog")))
-			languageElement->RemoveChild(dialog);
+		SetTextAttribute(languageElement, "date", "");
+	if (!cacheValid) {
+		pugi::xml_node dialog;
+		while ((dialog = languageElement.child("Dialog")))
+			languageElement.remove_child(dialog);
 	}
 
-	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
-	{
+	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
 		m_use_cache = cacheValid;
 		return true;
 	}
@@ -1123,8 +1095,7 @@ void CWrapEngine::CheckLanguage()
 		m_wrapOnEveryChar = true;
 		m_noWrapChars = noWrapChars_Chinese;
 	}
-	else
-	{
+	else {
 		m_wrapOnEveryChar = false;
 		m_noWrapChars = 0;
 	}
